@@ -32,12 +32,16 @@
  *
  *	Core threading and system headers.
  *
- *	$Header: /cvsroot/aolserver/aolserver/include/nsthread.h,v 1.24.2.4 2004/11/20 01:54:04 dossy Exp $
+ *	$Header: /cvsroot/aolserver/aolserver/include/nsthread.h,v 1.31 2005/08/23 21:41:31 jgdavidson Exp $
  */
 
 #ifndef NSTHREAD_H
 #define NSTHREAD_H
 
+
+#include "nsattributes.h"
+
+
 #ifdef _WIN32
 #define NS_EXPORT		__declspec(dllexport)
 #define NS_IMPORT		__declspec(dllimport)
@@ -148,10 +152,11 @@ NS_EXTERN int closedir(DIR *dp);
 #define NS_OK              	0
 #define NS_ERROR         	(-1)
 #define NS_TIMEOUT       	(-2)
+#define NS_BREAK	       	(-3)
 #define NS_THREAD_DETACHED	1
 #define NS_THREAD_JOINED	2
 #define NS_THREAD_EXITED	4
-#define NS_THREAD_NAMESIZE	32
+#define NS_THREAD_NAMESIZE	64
 #define NS_THREAD_MAXTLS	100
 
 /*
@@ -194,12 +199,12 @@ NS_EXTERN void Ns_MasterUnlock(void);
  * memory.c:
  */
 
-NS_EXTERN void *ns_malloc(size_t size);
-NS_EXTERN void *ns_calloc(size_t num, size_t size);
+NS_EXTERN void *ns_malloc(size_t size) _nsmalloc;
+NS_EXTERN void *ns_calloc(size_t num, size_t size) _nsmalloc;
 NS_EXTERN void ns_free(void *buf);
 NS_EXTERN void *ns_realloc(void *buf, size_t size);
-NS_EXTERN char *ns_strdup(const char *string);
-NS_EXTERN char *ns_strcopy(const char *string);
+NS_EXTERN char *ns_strdup(const char *string) _nsmalloc;
+NS_EXTERN char *ns_strcopy(const char *string) _nsmalloc;
 
 /*
  * mutex.c:
@@ -280,6 +285,7 @@ NS_EXTERN int ns_signal(int sig, void (*proc)(int));
  * thread.c:
  */
 
+NS_EXTERN void NsThreads_LibInit(void);
 NS_EXTERN void Ns_ThreadCreate(Ns_ThreadProc *proc, void *arg, long stackSize,
 			    Ns_Thread *resultPtr);
 NS_EXTERN void Ns_ThreadExit(void *arg);
@@ -292,6 +298,7 @@ NS_EXTERN char *Ns_ThreadGetName(void);
 NS_EXTERN char *Ns_ThreadGetParent(void);
 NS_EXTERN long Ns_ThreadStackSize(long size);
 NS_EXTERN void Ns_ThreadList(Tcl_DString *dsPtr, Ns_ThreadArgProc *proc);
+NS_EXTERN int Ns_CheckStack(void);
 
 /*
  * time.c:
