@@ -34,7 +34,7 @@
  *Support for pre-bound privileged ports.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/binder.c,v 1.15 2003/03/07 18:08:14 vasiljevic Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/binder.c,v 1.18 2005/08/17 21:18:21 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -50,13 +50,12 @@ static Ns_Mutex lock;
 /*
  *----------------------------------------------------------------------
  *
- * Ns_SockListenEx --
+ * NsSockGetBound --
  *
- *	Create a new socket bound to the specified port and listening
- *	for new connections.
+ *	Get a pre-bound socket if available.
  *
  * Results:
- *	Socket descriptor or -1 on error.
+ *	Socket descriptor or INVALID_SOCKET on error.
  *
  * Side effects:
  *	None.
@@ -64,32 +63,19 @@ static Ns_Mutex lock;
  *----------------------------------------------------------------------
  */
 
-int
-Ns_SockListenEx(char *address, int port, int backlog)
+SOCKET
+NsSockGetBound(struct sockaddr_in *saPtr)
 {
-    int err, sock = -1;
-    struct sockaddr_in sa;
+    SOCKET sock = INVALID_SOCKET;
     Tcl_HashEntry *hPtr;
 
-    if (Ns_GetSockAddr(&sa, address, port) != NS_OK) {
-	return -1;
-    }
     Ns_MutexLock(&lock);
-    hPtr = Tcl_FindHashEntry(&prebound, (char *) &sa);
+    hPtr = Tcl_FindHashEntry(&prebound, (char *) saPtr);
     if (hPtr != NULL) {
 	sock = (int) Tcl_GetHashValue(hPtr);
 	Tcl_DeleteHashEntry(hPtr);
     }
     Ns_MutexUnlock(&lock);
-    if (hPtr == NULL) {
-    	sock = Ns_SockBind(&sa);
-    }
-    if (sock != -1 && listen(sock, backlog) != 0) {
-	err = errno;
-	close(sock);
-	errno = err;
-	sock = -1;
-    }
     return sock;
 }
 
@@ -113,6 +99,7 @@ Ns_SockListenEx(char *address, int port, int backlog)
 void
 NsInitBinder(void)
 {
+    Ns_MutexSetName(&lock, "ns:binder");
     Tcl_InitHashTable(&prebound, sizeof(struct sockaddr_in)/sizeof(int));
 }
 
