@@ -30,34 +30,25 @@
 /*
  * compress.c --
  *
- * Support for simple gzip compression using Zlib.
+ *	Support for loadable gzip compression.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/compress.c,v 1.1.2.3 2005/01/12 19:11:46 dossy Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/compress.c,v 1.8 2006/06/26 00:28:09 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
-#include "ns.h"
+#include "nsd.h"
 
-#ifdef HAVE_ZLIB_H
-#include <zlib.h>
-
-static char header[] = {
-    037, 0213,  /* GZIP magic number. */
-    010,        /* Z_DEFLATED */
-    0,          /* flags */
-    0,0,0,0,    /* timestamp */
-    0,          /* xflags */
-    03};        /* Unix OS_CODE */
+static Ns_GzipProc *gzipProcPtr;
 
 
 /*
  *----------------------------------------------------------------------
  *
- * Ns_CompressGzip --
+ * Ns_Gzip --
  *
- *      Compress a string using gzip with RFC 1952 header/footer.
+ *      Compress a string.
  *
  * Results:
- *      NS_OK if compression worked, NS_ERROR otherwise.
+ *      Result of external compress proc, if any, otherwise NS_ERROR.
  *
  * Side effects:
  *      Will write compressed content to given Tcl_DString.
@@ -66,77 +57,63 @@ static char header[] = {
  */
 
 int
-Ns_CompressGzip(char *buf, int len, Tcl_DString *outPtr, int level)
+Ns_Gzip(char *buf, int len, int level, Tcl_DString *dsPtr)
 {
-    uLongf glen;
-    char *gbuf;
-    uLong crc;
-    int skip;
-    uint32_t footer[2];
-
-    /*
-     * Grow buffer for header, footer, and maximum compressed size.
-     */
-
-    glen = len + (len / 100) + 13 + sizeof(header) + sizeof(footer);
-    Tcl_DStringSetLength(outPtr, glen);
-
-    /*
-     * Compress output starting 2-bytes from the end of the header. 
-     */
-
-    gbuf = outPtr->string;
-    skip = sizeof(header) - 2;
-    glen -= skip;
-    if (compress2(gbuf + skip, &glen, buf, (uLong) len, level) != Z_OK) {
-        return NS_ERROR;
+    if (gzipProcPtr != NULL) {
+	return (*gzipProcPtr)(buf, len, level, dsPtr);
     }
-    memcpy(gbuf, header, sizeof(header));
-    Tcl_DStringSetLength(outPtr, glen + skip);
-
-    /*
-     * Append footer of CRC and uncompressed length.
-     */
-
-    crc = crc32(0, Z_NULL, 0);
-    crc = crc32(crc, buf, len);
-    footer[0] = htonl(crc);
-    footer[1] = htonl(len);
-    Tcl_DStringAppend(outPtr, (char *) footer, sizeof(footer));
-
-    return NS_OK;
-}
-
-#else
-
-int
-Ns_CompressGzip(char *buf, int len, Tcl_DString *outPtr, int level)
-{
     return NS_ERROR;
 }
 
-#endif
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Ns_SetGzipProc --
+ *
+ *      Set the global procedure for compression.  Called by the
+ *	nszlib module when loaded.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Later calls to Ns_Gzip will use given function.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+Ns_SetGzipProc(Ns_GzipProc *procPtr)
+{
+    gzipProcPtr = procPtr;
+}
 
 
 /*
  *----------------------------------------------------------------------
  *
- * Ns_Compress --
+ * Ns_Compress, Ns_CompressGzip --
  *
- *      Deprecated:  Used by nsjk2 module.  Remove this once nsjk2 has
- *      been updated to use Ns_CompressGzip directly.
+ *      AOLserver 4.0 compression routines.
  *
  * Results:
- *      NS_OK if compression worked, NS_ERROR otherwise.
+ *      See Ns_Gzip.
  *
  * Side effects:
- *      Will write compressed content to given Tcl_DString.
+ *      See Ns_Gzip.
  *
  *----------------------------------------------------------------------
  */
+
 int
 Ns_Compress(char *buf, int len, Tcl_DString *outPtr, int level)
 {
-    return Ns_CompressGzip(buf, len, outPtr, level);
+    return Ns_Gzip(buf, len, level, outPtr);
 }
 
+int
+Ns_CompressGzip(char *buf, int len, Tcl_DString *outPtr, int level)
+{
+    return Ns_Gzip(buf, len, level, outPtr);
+}
