@@ -34,7 +34,7 @@
  *      Handle connection I/O.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/connio.c,v 1.12.2.2 2004/09/17 15:58:19 dossy Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/connio.c,v 1.27 2006/04/19 17:48:43 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 #define IOBUFSZ 2048
@@ -44,9 +44,9 @@ static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/conni
  */
 
 static int ConnSend(Ns_Conn *conn, int nsend, Tcl_Channel chan,
-    	    	    FILE *fp, int fd);
-static int ConnCopy(Ns_Conn *conn, size_t ncopy, Tcl_Channel chan,
-		    FILE *fp, int fd);
+        FILE *fp, int fd, off_t off);
+static int ConnCopy(Ns_Conn *conn, size_t ncopy, Ns_DString *dsPtr,
+        Tcl_Channel chan, FILE *fp, int fd);
  
 
 /*
@@ -54,11 +54,11 @@ static int ConnCopy(Ns_Conn *conn, size_t ncopy, Tcl_Channel chan,
  *
  * Ns_ConnInit --
  *
- *	Initialize a connection (no longer used).
+ *	Initialize a connection, currently doing nothing.
  *
  * Results:
  *	Always NS_OK.
- * 
+ *
  * Side effects:
  *	None.
  *
@@ -94,17 +94,193 @@ Ns_ConnClose(Ns_Conn *conn)
     int		      keep;
     
     if (connPtr->sockPtr != NULL) {
+	Ns_GetTime(&connPtr->times.close);
 	keep = (conn->flags & NS_CONN_KEEPALIVE) ? 1 : 0;
 	NsSockClose(connPtr->sockPtr, keep);
 	connPtr->sockPtr = NULL;
 	connPtr->flags |= NS_CONN_CLOSED;
-	if (connPtr->interp != NULL) {
-	    NsRunAtClose(connPtr->interp);
+	if (connPtr->itPtr != NULL) {
+	    NsTclRunAtClose(connPtr->itPtr);
 	}
     }
     return NS_OK;
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Ns_ConnFlush --
+ *
+ *	Flush the headers and/or response content, handling character
+ *	encoding and/or gzip compression if necessary.
+ *
+ * Results:
+ *	NS_ERROR if a connection write routine failed, NS_OK otherwise.
+ *
+ * Side effects:
+ *  	Content may be encoded and/or gzip'ed before calling
+ *	Ns_ConnFlushDirect.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Ns_ConnFlush(Ns_Conn *conn, char *buf, int len, int stream)
+{
+    Conn *connPtr = (Conn *) conn;
+    NsServer *servPtr = connPtr->servPtr;
+    Tcl_Encoding encoding;
+    Tcl_DString  enc, gzip;
+    char *ahdr;
+    int status;
+
+    Tcl_DStringInit(&enc);
+    Tcl_DStringInit(&gzip);
+    if (len < 0) {
+	len = strlen(buf);
+    }
+
+    /*
+     * Encode content to the expected charset.
+     */
+
+    encoding = Ns_ConnGetEncoding(conn);
+    if (encoding != NULL) {
+	Tcl_UtfToExternalDString(encoding, buf, len, &enc);
+	buf = enc.string;
+	len = enc.length;
+    }
+
+    /*
+     * GZIP the content when not streaming if enabled and the content
+     * length is above the minimum.
+     */
+
+    if (!stream
+	    && (conn->flags & NS_CONN_GZIP)
+	    && (servPtr->opts.flags & SERV_GZIP)
+	    && (len > (int) servPtr->opts.gzipmin)
+	    && (ahdr = Ns_SetIGet(conn->headers, "Accept-Encoding")) != NULL
+	    && strstr(ahdr, "gzip") != NULL
+	    && Ns_Gzip(buf, len, servPtr->opts.gziplevel, &gzip) == NS_OK) {
+	buf = gzip.string;
+	len = gzip.length;
+	Ns_ConnCondSetHeaders(conn, "Content-Encoding", "gzip");
+    }
+
+    /*
+     * Flush content.
+     */
+
+    status = Ns_ConnFlushDirect(conn, buf, len, stream);
+    Tcl_DStringFree(&enc);
+    Tcl_DStringFree(&gzip);
+    return status;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Ns_ConnFlushDirect --
+ *
+ *	Flush the headers and/or response content either as a single
+ *	response or in stream/chunked mode.
+ *
+ * Results:
+ *	NS_ERROR if a connection write routine failed, NS_OK otherwise.
+ *
+ * Side effects:
+ *  	Headers will be flushed on first write.  Output will be
+ *	chunked if streaming to an HTTP version 1.1 or greater client.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Ns_ConnFlushDirect(Ns_Conn *conn, char *buf, int len, int stream)
+{
+    struct iovec iov[4];
+    int i, nwrote, towrite, hlen, ioc;
+    char hdr[100];
+
+    /*
+     * Queue headers if not already sent.
+     */
+
+    if (!(conn->flags & NS_CONN_SENTHDRS)) {
+	if (!stream) {
+	    hlen = len;
+	} else {
+	    if (conn->request->version > 1.0) {
+		conn->flags |= NS_CONN_CHUNK;
+	    }
+	    hlen = -1;	/* NB: Surpress Content-length header. */
+	}
+	Ns_ConnSetRequiredHeaders(conn, Ns_ConnGetType(conn), hlen);
+	if (conn->flags & NS_CONN_CHUNK) {
+	    Ns_ConnCondSetHeaders(conn, "Transfer-Encoding", "chunked");
+	}
+	Ns_ConnQueueHeaders(conn, Ns_ConnGetStatus(conn));
+    }
+
+    /*
+     * Send content on any request other than HEAD.
+     */
+
+    ioc = 0;
+    towrite = 0;
+    if (!(conn->flags & NS_CONN_SKIPBODY)) {
+    	if (!(conn->flags & NS_CONN_CHUNK)) {
+	    /*
+	     * Output content without chunking header/trailers.
+	     */
+
+    	    iov[ioc].iov_base = buf;
+    	    iov[ioc++].iov_len = len;
+        } else {
+	    if (len > 0) {
+		/*
+		 * Output length header followed by content and trailer.
+		 */
+
+    	        iov[ioc].iov_base = hdr;
+	        iov[ioc++].iov_len = sprintf(hdr, "%x\r\n", len);
+    	        iov[ioc].iov_base = buf;
+    	        iov[ioc++].iov_len = len;
+	        iov[ioc].iov_base = "\r\n";
+	        iov[ioc++].iov_len = 2;
+	    }
+	    if (!stream) {
+		/*
+		 * Output end-of-content trailer.
+		 */
+
+    	        iov[ioc].iov_base = "0\r\n\r\n";
+	        iov[ioc++].iov_len = 5;
+	    }
+	}
+    	for (i = 0; i < ioc; ++i) {
+	    towrite += iov[i].iov_len;
+    	}
+    }
+
+    /*
+     * Write the output buffer and if not streaming, close the
+     * connection.
+     */
+
+    nwrote = Ns_ConnSend(conn, iov, ioc);
+    if (nwrote != towrite) {
+    	return NS_ERROR;
+    }
+    if (!stream && Ns_ConnClose(conn) != NS_OK) {
+	return NS_ERROR;
+    }
+    return NS_OK;
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -132,10 +308,6 @@ Ns_ConnSend(Ns_Conn *conn, struct iovec *bufs, int nbufs)
     int         nwrote, towrite, i, n;
     struct iovec    sbufs[16];
 
-    if (connPtr->sockPtr == NULL) {
-	return -1;
-    }
-
     /*
      * Send up to 16 buffers, including the queued output
      * buffer if necessary.
@@ -143,9 +315,9 @@ Ns_ConnSend(Ns_Conn *conn, struct iovec *bufs, int nbufs)
 
     towrite = 0;
     n = 0;
-    if (connPtr->queued.length > 0) {
-	sbufs[n].iov_base = connPtr->queued.string;
-	sbufs[n].iov_len = connPtr->queued.length;
+    if (connPtr->obuf.length > 0) {
+	sbufs[n].iov_base = connPtr->obuf.string;
+	sbufs[n].iov_len = connPtr->obuf.length;
 	towrite += sbufs[n].iov_len;
 	++n;
     }
@@ -161,7 +333,7 @@ Ns_ConnSend(Ns_Conn *conn, struct iovec *bufs, int nbufs)
     bufs = sbufs;
     n = nwrote = 0;
     while (towrite > 0) {
-	n = NsSockSend(connPtr->sockPtr, bufs, nbufs);
+	n = NsConnSend(conn, bufs, nbufs);
 	if (n < 0) {
 	    break;
 	}
@@ -183,15 +355,15 @@ Ns_ConnSend(Ns_Conn *conn, struct iovec *bufs, int nbufs)
     }
     if (nwrote > 0) {
         connPtr->nContentSent += nwrote;
-	if (connPtr->queued.length > 0) {
-	    n = connPtr->queued.length - nwrote;
+	if (connPtr->obuf.length > 0) {
+	    n = connPtr->obuf.length - nwrote;
 	    if (n <= 0) {
-		nwrote -= connPtr->queued.length;
-		Tcl_DStringTrunc(&connPtr->queued, 0);
+		nwrote -= connPtr->obuf.length;
+		Tcl_DStringTrunc(&connPtr->obuf, 0);
 	    } else {
-		memmove(connPtr->queued.string,
-		    connPtr->queued.string + nwrote, (size_t)n);
-		Tcl_DStringTrunc(&connPtr->queued, n);
+		memmove(connPtr->obuf.string,
+		    connPtr->obuf.string + nwrote, (size_t)n);
+		Tcl_DStringTrunc(&connPtr->obuf, n);
 		nwrote = 0;
 	    }
 	}
@@ -295,59 +467,19 @@ Ns_WriteConn(Ns_Conn *conn, char *buf, int len)
 int
 Ns_WriteCharConn(Ns_Conn *conn, char *buf, int len)
 {
-    int             status;
-    char           *utfBytes;
-    int             utfCount; /* # of bytes in utfBytes */
-    int             utfConvertedCount;  /* # of bytes of utfBytes converted */
-    char            encodedBytes[IOBUFSZ];
-    int             encodedCount; /* # of bytes converted in encodedBytes */
-    Tcl_Interp     *interp;
-    Conn           *connPtr = (Conn *)conn;
+    Tcl_Encoding    encoding;
+    Tcl_DString	    enc;
+    int		    status;
 
-    status = NS_OK;
-
-    if (connPtr->encoding == NULL) {
-
-	status = Ns_WriteConn(conn, buf, len);
-
-    } else {
-
-	utfBytes = buf;
-	utfCount = len;
-	interp = Ns_GetConnInterp(conn);
-
-	while (utfCount > 0 && status == NS_OK) {
-
-	    /* Convert a chunk to the desired encoding. */
-
-	    status = Tcl_UtfToExternal(interp,
-		connPtr->encoding,
-		utfBytes, utfCount,
-		0, NULL,              /* flags, encoding state */
-		encodedBytes, sizeof(encodedBytes),
-		&utfConvertedCount,
-		&encodedCount,
-		NULL                  /* # of chars encoded */
-	    );
-
-	    if (status != TCL_OK && status != TCL_CONVERT_NOSPACE) {
-		status = NS_ERROR;
-		break;
-	    }
-
-	    /* Send the converted chunk. */
-
-	    status = NS_OK;
-	    buf = encodedBytes;
-	    len = encodedCount;
-
-	    status = Ns_WriteConn(conn, buf, len);
-
-	    utfCount -= utfConvertedCount;
-	    utfBytes += utfConvertedCount;
-	}
+    Tcl_DStringInit(&enc);
+    encoding = Ns_ConnGetEncoding(conn);
+    if (encoding != NULL) {
+	Tcl_UtfToExternalDString(encoding, buf, len, &enc);
+	buf = enc.string;
+	len = enc.length;
     }
-
+    status = Ns_WriteConn(conn, buf, len);
+    Tcl_DStringFree(&enc);
     return status;
 }
 
@@ -418,19 +550,25 @@ Ns_ConnSendDString(Ns_Conn *conn, Ns_DString *dsPtr)
 int
 Ns_ConnSendChannel(Ns_Conn *conn, Tcl_Channel chan, int nsend)
 {
-    return ConnSend(conn, nsend, chan, NULL, -1);
+    return ConnSend(conn, nsend, chan, NULL, -1, -1);
 }
 
 int
 Ns_ConnSendFp(Ns_Conn *conn, FILE *fp, int nsend)
 {
-    return ConnSend(conn, nsend, NULL, fp, -1);
+    return ConnSend(conn, nsend, NULL, fp, -1, -1);
 }
 
 int
 Ns_ConnSendFd(Ns_Conn *conn, int fd, int nsend)
 {
-    return ConnSend(conn, nsend, NULL, NULL, fd);
+    return ConnSend(conn, nsend, NULL, NULL, fd, -1);
+}
+
+int
+Ns_ConnSendFdEx(Ns_Conn *conn, int fd, off_t off, int nsend)
+{
+    return ConnSend(conn, nsend, NULL, NULL, fd, off);
 }
 
 
@@ -453,14 +591,12 @@ Ns_ConnSendFd(Ns_Conn *conn, int fd, int nsend)
 int
 Ns_ConnFlushContent(Ns_Conn *conn)
 {
-    Conn *connPtr = (Conn *) conn;
-    Request *reqPtr = connPtr->reqPtr;
+    int avail;
 
-    if (connPtr->sockPtr == NULL) {
-	return -1;
+    if (NsConnContent(conn, NULL, &avail) == NULL) {
+	return NS_ERROR;
     }
-    reqPtr->next  += reqPtr->avail;
-    reqPtr->avail  = 0;
+    NsConnSeek(conn, avail);
     return NS_OK;
 }
 
@@ -521,18 +657,17 @@ Ns_ConnGets(char *buf, size_t bufsize, Ns_Conn *conn)
 int
 Ns_ConnRead(Ns_Conn *conn, void *vbuf, int toread)
 {
-    Conn *connPtr = (Conn *) conn;
-    Request *reqPtr = connPtr->reqPtr;
+    int avail;
+    char *next;
 
-    if (connPtr->sockPtr == NULL) {
+    if (NsConnContent(conn, &next, &avail) == NULL) {
 	return -1;
     }
-    if (toread > reqPtr->avail) {
-	toread = reqPtr->avail;
+    if (toread > avail) {
+	toread = avail;
     }
-    memcpy(vbuf, reqPtr->next, (size_t)toread);
-    reqPtr->next  += toread;
-    reqPtr->avail -= toread;
+    memcpy(vbuf, next, (size_t) toread);
+    NsConnSeek(conn, toread);
     return toread;
 }
 
@@ -557,14 +692,19 @@ int
 Ns_ConnReadLine(Ns_Conn *conn, Ns_DString *dsPtr, int *nreadPtr)
 {
     Conn	   *connPtr = (Conn *) conn;
-    Request	   *reqPtr = connPtr->reqPtr;
-    NsServer	   *servPtr = connPtr->servPtr;
-    char           *eol;
-    int             nread, ncopy;
+    Driver         *drvPtr = connPtr->drvPtr;
+    char           *eol, *next;
+    int             nread, ncopy, avail;
 
-    if (connPtr->sockPtr == NULL
-	|| (eol = strchr(reqPtr->next, '\n')) == NULL
-	|| (nread = (eol - reqPtr->next)) > servPtr->limits.maxline) {
+    if (NsConnContent(conn, &next, &avail) == NULL) {
+	return NS_ERROR;
+    }
+    eol = memchr(next, '\n', avail);
+    if (eol == NULL) {
+	eol = next + avail;
+    }
+    nread = eol - next;
+    if (nread > drvPtr->maxline) {
 	return NS_ERROR;
     }
     ncopy = nread;
@@ -575,10 +715,9 @@ Ns_ConnReadLine(Ns_Conn *conn, Ns_DString *dsPtr, int *nreadPtr)
     if (ncopy > 0 && eol[-1] == '\r') {
 	--ncopy;
     }
-    Ns_DStringNAppend(dsPtr, reqPtr->next, ncopy);
-    reqPtr->next  += nread;
-    reqPtr->avail -= nread;
-    return NS_ERROR;
+    Ns_DStringNAppend(dsPtr, next, ncopy);
+    NsConnSeek(conn, nread);
+    return NS_OK;
 }
 
 
@@ -608,7 +747,7 @@ Ns_ConnReadHeaders(Ns_Conn *conn, Ns_Set *set, int *nreadPtr)
 
     Ns_DStringInit(&ds);
     nread = 0;
-    maxhdr = servPtr->limits.maxheaders;
+    maxhdr = connPtr->drvPtr->maxinput;
     status = NS_OK;
     while (nread < maxhdr && status == NS_OK) {
         Ns_DStringTrunc(&ds, 0);
@@ -636,68 +775,41 @@ Ns_ConnReadHeaders(Ns_Conn *conn, Ns_Set *set, int *nreadPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Ns_ConnCopyToDString --
+ * Ns_ConnCopyToDString, File, Fd, Channel --
  *
- *	Copy data from a connection to a dstring.
+ *      Copy data from a connection to a DString, channel, FILE, or fd. 
  *
  * Results:
  *	NS_OK or NS_ERROR 
  *
  * Side effects:
- *	None.
+ *      See ConnCopy().
  *
  *----------------------------------------------------------------------
  */
 
 int
-Ns_ConnCopyToDString(Ns_Conn *conn, size_t tocopy, Ns_DString *dsPtr)
+Ns_ConnCopyToDString(Ns_Conn *conn, size_t ncopy, Ns_DString *dsPtr)
 {
-    Conn *connPtr = (Conn *) conn;
-    Request *reqPtr = connPtr->reqPtr;
-    int ncopy = (int) tocopy;
-
-    if (connPtr->sockPtr == NULL || reqPtr->avail < ncopy) {
-	return NS_ERROR;
-    }
-    Ns_DStringNAppend(dsPtr, reqPtr->next, ncopy);
-    reqPtr->next  += ncopy;
-    reqPtr->avail -= ncopy;
-    return NS_OK;
+    return ConnCopy(conn, ncopy, dsPtr, NULL, NULL, -1);
 }
 
-
-/*
- *----------------------------------------------------------------------
- *
- * Ns_ConnCopyToFile, Fd, Channel --
- *
- *	Copy data from a connection to a channel, FILE, or fd. 
- *
- * Results:
- *	NS_OK or NS_ERROR 
- *
- * Side effects:
- *	See ConnCopy().
- *
- *----------------------------------------------------------------------
- */
-
 int
 Ns_ConnCopyToChannel(Ns_Conn *conn, size_t ncopy, Tcl_Channel chan)
 {
-    return ConnCopy(conn, ncopy, chan, NULL, -1);
+    return ConnCopy(conn, ncopy, NULL, chan, NULL, -1);
 }
 
 int
 Ns_ConnCopyToFile(Ns_Conn *conn, size_t ncopy, FILE *fp)
 {
-    return ConnCopy(conn, ncopy, NULL, fp, -1);
+    return ConnCopy(conn, ncopy, NULL, NULL, fp, -1);
 }
 
 int
 Ns_ConnCopyToFd(Ns_Conn *conn, size_t ncopy, int fd)
 {
-    return ConnCopy(conn, ncopy, NULL, NULL, fd);
+    return ConnCopy(conn, ncopy, NULL, NULL, NULL, fd);
 }
 
 
@@ -706,7 +818,7 @@ Ns_ConnCopyToFd(Ns_Conn *conn, size_t ncopy, int fd)
  *
  * ConnCopy --
  *
- *	Copy connection content to a channel, FILE, or fd.
+ *      Copy connection content to a DString, channel, FILE, or fd.
  *
  * Results:
  *  	NS_OK or NS_ERROR if not all content could be read.
@@ -718,33 +830,36 @@ Ns_ConnCopyToFd(Ns_Conn *conn, size_t ncopy, int fd)
  */
 
 static int
-ConnCopy(Ns_Conn *conn, size_t tocopy, Tcl_Channel chan, FILE *fp, int fd)
+ConnCopy(Ns_Conn *conn, size_t tocopy, Ns_DString *dsPtr, Tcl_Channel chan,
+    FILE *fp, int fd)
 {
-    Conn       *connPtr = (Conn *) conn;
-    Request    *reqPtr = connPtr->reqPtr;
-    int		nwrote;
+    int		nwrote, avail;
     int		ncopy = (int) tocopy;
+    char       *next;
 
-    if (connPtr->sockPtr == NULL || reqPtr->avail < ncopy) {
+    if (NsConnContent(conn, &next, &avail) == NULL || avail < ncopy) {
 	return NS_ERROR;
     }
     while (ncopy > 0) {
-	if (chan != NULL) {
-	    nwrote = Tcl_Write(chan, reqPtr->next, ncopy);
+        if (dsPtr != NULL) {
+            Ns_DStringNAppend(dsPtr, next, ncopy);
+            nwrote = ncopy;
+        } else if (chan != NULL) {
+	    nwrote = Tcl_Write(chan, next, ncopy);
     	} else if (fp != NULL) {
-            nwrote = fwrite(reqPtr->next, 1, (size_t)ncopy, fp);
+            nwrote = fwrite(next, 1, (size_t) ncopy, fp);
             if (ferror(fp)) {
 		nwrote = -1;
 	    }
 	} else {
-	    nwrote = write(fd, reqPtr->next, (size_t)ncopy);
+	    nwrote = write(fd, next, (size_t)ncopy);
 	}
 	if (nwrote < 0) {
 	    return NS_ERROR;
 	}
 	ncopy -= nwrote;
-	reqPtr->next  += nwrote;
-	reqPtr->avail -= nwrote;
+	next  += nwrote;
+	NsConnSeek(conn, nwrote);
     }
     return NS_OK;
 }
@@ -767,35 +882,47 @@ ConnCopy(Ns_Conn *conn, size_t tocopy, Tcl_Channel chan, FILE *fp, int fd)
  */
 
 static int
-ConnSend(Ns_Conn *conn, int nsend, Tcl_Channel chan, FILE *fp, int fd)
+ConnSend(Ns_Conn *conn, int nsend, Tcl_Channel chan, FILE *fp, int fd,
+	 off_t off)
 {
-    int             toread, nread, status;
+    size_t	    toread;
+    int             nread, status;
     char            buf[IOBUFSZ];
 
     /*
      * Even if nsend is 0, ensure all queued data (like HTTP response
      * headers) get flushed.
      */
+
     if (nsend == 0) {
         Ns_WriteConn(conn, NULL, 0);
     }
 
     status = NS_OK;
     while (status == NS_OK && nsend > 0) {
-        toread = nsend;
+        toread = (size_t) nsend;
         if (toread > sizeof(buf)) {
             toread = sizeof(buf);
         }
 	if (chan != NULL) {
-	    nread = Tcl_Read(chan, buf, toread);
+	    nread = Tcl_Read(chan, buf, (int) toread);
 	} else if (fp != NULL) {
-            nread = fread(buf, 1, (size_t)toread, fp);
+            nread = fread(buf, 1, toread, fp);
             if (ferror(fp)) {
 	    	nread = -1;
 	    }
-    	} else {
-	    nread = read(fd, buf, (size_t)toread);
-    	}
+    	} else if (off < 0) {
+	    nread = read(fd, buf, toread);
+	} else {
+#ifdef WIN32
+	    nread = -1;
+#else
+	    nread = pread(fd, buf, toread, off);
+	    if (nread > 0) {
+		off += (off_t) nread;
+	    }
+#endif
+	}
 	if (nread == -1) {
 	    status = NS_ERROR;
 	} else if (nread == 0) { 
@@ -804,6 +931,5 @@ ConnSend(Ns_Conn *conn, int nsend, Tcl_Channel chan, FILE *fp, int fd)
             nsend -= nread;
 	}
     }
-
     return status;
 }
