@@ -34,7 +34,7 @@
  *
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/driver.c,v 1.55 2006/06/26 00:28:18 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/driver.c,v 1.58 2009/01/29 20:54:24 asteets Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -946,7 +946,7 @@ DriverThread(void *arg)
     drvPtr->flags |= flags;
     Ns_CondBroadcast(&drvPtr->cond);
     Ns_MutexUnlock(&drvPtr->lock);
-    
+
     /*
      * Loop forever until signalled to shutdown and all
      * connections are complete and gracefully closed.
@@ -1307,7 +1307,7 @@ dropped:
 	    Tcl_DStringStartSublist(drvPtr->queryPtr);
 	    Ns_DStringPrintf(drvPtr->queryPtr,
 		"time %ld:%ld "
-		"spins %ld accepts %u queued %u reads %u "
+		"spins %d accepts %u queued %u reads %u "
 		"dropped %u overflow %d timeout %u",
 	    	now.sec, now.usec,
 		drvPtr->stats.spins, drvPtr->stats.accepts,
@@ -1328,7 +1328,12 @@ dropped:
 		    pdata.pfds[sockPtr->pidx].revents, 
 		    sockPtr->acceptTime.sec, sockPtr->acceptTime.usec,
 		    sockPtr->timeout.sec, sockPtr->timeout.usec);
-	    	NsAppendConn(drvPtr->queryPtr, sockPtr->connPtr, "i/o");
+		if (sockPtr->connPtr != NULL) {
+		    NsAppendConn(drvPtr->queryPtr, sockPtr->connPtr, "i/o");
+		} else {
+		    Tcl_DStringStartSublist(drvPtr->queryPtr);
+		    Tcl_DStringEndSublist(drvPtr->queryPtr);
+		}
 	    	Tcl_DStringEndSublist(drvPtr->queryPtr);
 		sockPtr = sockPtr->nextPtr;
 	    }
