@@ -33,30 +33,22 @@
  *	Defines standard default charset to encoding mappings.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/encoding.c,v 1.13 2003/03/07 18:08:25 vasiljevic Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/encoding.c,v 1.17 2005/08/01 20:27:35 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
-
-#define DEFAULT_CHARSET_CONFIG    "OutputCharset"
-#define HACK_CONTENT_TYPE_CONFIG  "HackContentType"
-
 /*
  * Local functions defined in this file.
  */
 
 static void AddExtension(char *name, char *charset);
 static void AddCharset(char *name, char *charset);
-static Tcl_Encoding GetCharsetEncoding(char *charset, int len);
-static char *RebuildCharsetList(void);
-static int GetDefaultHackContentTypeP(void);
-static Tcl_Encoding GetDefaultCharset(void);
-static Tcl_Encoding GetDefaultEncoding(void);
 
 /*
  * Static variables defined in this file.
  */
 
+static int		eid;
 static Tcl_HashTable    encodings;
 static Tcl_HashTable    charsets;
 static Tcl_HashTable    extensions;
@@ -64,13 +56,6 @@ static Ns_Mutex		lock;
 static Ns_Cond		cond;
 #define ENC_LOCKED	((Tcl_Encoding) (-1))
 
-static char            *charsetList = NULL;
-/*
-static Tcl_Encoding     defaultEncoding;
-static char            *defaultCharset;
-static int              hackContentTypeP;
-*/
-
 /*
  * The default table maps file extension to Tcl encodings.
  */
@@ -161,6 +146,92 @@ static struct {
     { NULL,     NULL }
 };
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * NsInitEncodings --
+ *
+ *	Add compiled-in default encodings.
+ *
+ * Results:
+ *	None. 
+ *
+ * Side effects:
+ *	None. 
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+NsInitEncodings(void)
+{
+    int     i;
+
+    /*
+     * Allocate URL space id for input encodings.
+     */
+
+    eid = Ns_UrlSpecificAlloc();
+
+    /*
+     * Initialize hash table of encodings and charsets.
+     */
+
+    Ns_MutexSetName(&lock, "ns:encodings");
+    Tcl_InitHashTable(&encodings, TCL_STRING_KEYS);
+    Tcl_InitHashTable(&charsets, TCL_STRING_KEYS);
+    Tcl_InitHashTable(&extensions, TCL_STRING_KEYS);
+
+    /*
+     * Add default charset and file mappings.
+     */
+
+    for (i = 0; builtinChar[i].charset != NULL; ++i) {
+        AddCharset(builtinChar[i].charset, builtinChar[i].name);
+    }
+    for (i = 0; builtinExt[i].extension != NULL; ++i) {
+	AddExtension(builtinExt[i].extension, builtinExt[i].name);
+    }
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * NsUpdateEncodings --
+ *
+ *	Add additional configured encodings.
+ *
+ * Results:
+ *	None. 
+ *
+ * Side effects:
+ *	None. 
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+NsUpdateEncodings(void)
+{
+    Ns_Set *set;
+    int     i;
+
+    /*
+     * Add configured charsets and file mappings.
+     */
+
+    set = Ns_ConfigGetSection("ns/charsets");
+    for (i = 0; set != NULL && i < Ns_SetSize(set); ++i) {
+	AddCharset(Ns_SetKey(set, i), Ns_SetValue(set, i));
+    }
+    set = Ns_ConfigGetSection("ns/encodings");
+    for (i = 0; set != NULL && i < Ns_SetSize(set); ++i) {
+	AddExtension(Ns_SetKey(set, i), Ns_SetValue(set, i));
+    }
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -224,15 +295,18 @@ Ns_GetFileEncoding(char *file)
 Tcl_Encoding
 Ns_GetTypeEncoding(char *type)
 {
-    int   used_default;
-    return NsGetTypeEncodingWithDef(type, &used_default);
+    char *charset;
+    int len;
+
+    charset = Ns_FindCharset(type, &len);
+    return (charset ? Ns_GetCharsetEncodingEx(charset, len) : NULL);
 }
 
 
 /*
  *----------------------------------------------------------------------
  *
- * Ns_GetCharsetEncoding --
+ * Ns_GetCharsetEncodingEx, Ns_GetCharsetEncoding --
  *
  *	Return the Tcl_Encoding for the given charset, e.g.,
  *	"iso-8859-1" returns Tcl_Encoding for iso8859-1.
@@ -246,12 +320,36 @@ Ns_GetTypeEncoding(char *type)
  *----------------------------------------------------------------------
  */
 
-
-
 Tcl_Encoding
 Ns_GetCharsetEncoding(char *charset)
 {
-    return GetCharsetEncoding(charset, -1);
+    return Ns_GetCharsetEncodingEx(charset, -1);
+}
+
+Tcl_Encoding
+Ns_GetCharsetEncodingEx(char *charset, int len)
+{
+    Tcl_HashEntry *hPtr;
+    Tcl_Encoding encoding;
+    Ns_DString ds;
+
+    /*
+     * Cleanup the charset name and check for an
+     * alias (e.g., iso-8859-1 = iso8859-1) before
+     * assuming the charset and Tcl encoding names
+     * match (e.g., big5).
+     */
+    
+    Ns_DStringInit(&ds);
+    Ns_DStringNAppend(&ds, charset, len);
+    charset = Ns_StrTrim(Ns_StrToLower(ds.string));
+    hPtr = Tcl_FindHashEntry(&charsets, charset);
+    if (hPtr != NULL) {
+	charset = Tcl_GetHashValue(hPtr);
+    }
+    encoding = Ns_GetEncoding(charset);
+    Ns_DStringFree(&ds);
+    return encoding;
 }
 
 
@@ -305,209 +403,101 @@ Ns_GetEncoding(char *name)
 /*
  *----------------------------------------------------------------------
  *
- * NsInitEncodings --
+ * Ns_FindCharset --
  *
- *	Add compiled-in default encodings.
+ *	Find start end length of charset within a type string.
  *
  * Results:
- *	None. 
+ *	Pointer to start of charset or NULL on no charset.
  *
  * Side effects:
- *	None. 
+ *	Will update lenPtr with length of charset if found.
  *
  *----------------------------------------------------------------------
  */
 
-void
-NsInitEncodings(void)
+char *
+Ns_FindCharset(char *type, int *lenPtr)
 {
-    int     i;
+    char *start, *end;
 
-    /*
-     * Initialize hash table of encodings and charsets.
-     */
-
-    Ns_MutexSetName(&lock, "ns:encodings");
-    Tcl_InitHashTable(&encodings, TCL_STRING_KEYS);
-    Tcl_InitHashTable(&charsets, TCL_STRING_KEYS);
-    Tcl_InitHashTable(&extensions, TCL_STRING_KEYS);
-
-    /*
-     * Add default charset and file mappings.
-     */
-
-    for (i = 0; builtinChar[i].charset != NULL; ++i) {
-        AddCharset(builtinChar[i].charset, builtinChar[i].name);
-    }
-    for (i = 0; builtinExt[i].extension != NULL; ++i) {
-	AddExtension(builtinExt[i].extension, builtinExt[i].name);
-    }
-
-    RebuildCharsetList();
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * NsUpdateEncodings --
- *
- *	Add additional configured encodings.
- *
- * Results:
- *	None. 
- *
- * Side effects:
- *	None. 
- *
- *----------------------------------------------------------------------
- */
-
-void
-NsUpdateEncodings(void)
-{
-    Ns_Set *set;
-    int     i;
-
-    /*
-     * Add configured charsets and file mappings.
-     */
-
-    set = Ns_ConfigGetSection("ns/charsets");
-    for (i = 0; set != NULL && i < Ns_SetSize(set); ++i) {
-	AddCharset(Ns_SetKey(set, i), Ns_SetValue(set, i));
-    }
-    set = Ns_ConfigGetSection("ns/encodings");
-    for (i = 0; set != NULL && i < Ns_SetSize(set); ++i) {
-	AddExtension(Ns_SetKey(set, i), Ns_SetValue(set, i));
-    }
-
-    RebuildCharsetList();
-
-
-    /*
-     * Establish default output encoding, if present.  If this
-     * configuration specification is not present, the default
-     * behavior will be to do not encoding transformation.
-     */
-    nsconf.encoding.outputCharset = Ns_ConfigGetValue(NS_CONFIG_PARAMETERS,
-                                       DEFAULT_CHARSET_CONFIG);
-    if (nsconf.encoding.outputCharset != NULL) {
-
-        nsconf.encoding.outputEncoding =
-            Ns_GetCharsetEncoding(nsconf.encoding.outputCharset);
-        if (nsconf.encoding.outputEncoding == NULL) {
-            Ns_Fatal("could not find encoding for default output charset \"%s\"",
-                     nsconf.encoding.outputCharset);
-        }
-
-        /* Initialize hackContentTypeP. */
-
-        nsconf.encoding.hackContentTypeP = NS_TRUE;
-        Ns_ConfigGetBool(NS_CONFIG_PARAMETERS, HACK_CONTENT_TYPE_CONFIG,
-                         &nsconf.encoding.hackContentTypeP);
-    } else {
-        nsconf.encoding.outputEncoding = NULL;
-        nsconf.encoding.hackContentTypeP = NS_FALSE;
-    }
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * NsGetTypeEncodingWithDef --
- *
- *	Return the Tcl_Encoding for the given Content-type header,
- *	e.g., "text/html; charset=iso-8859-1" returns Tcl_Encoding
- *	for iso8859-1.
- *      This function will utilize the ns/parameters/OutputCharset
- *      config parameter if given a content-type "text/<anything>" with
- *      no charset.
- *      When no OutputCharset defined, the fall-back behavior is to
- *      return NULL.
- *      The used_default output parameter will be set to NS_TRUE when
- *      OutputCharset was used to determine the encoding.
- *
- * Results:
- *	Tcl_Encoding or NULL if not found.
- *
- * Side effects:
- *	See LoadEncoding().
- *
- *----------------------------------------------------------------------
- */
-
-Tcl_Encoding
-NsGetTypeEncodingWithDef(char *type, int *used_default)
-{
-    char *s, *e;
-    Tcl_Encoding ret_enc = NULL;
-
-    s = Ns_StrCaseFind(type, "charset");
-    if (s != NULL) {
-	s += 7;
-	s += strspn(s, " ");
-        *used_default = NS_FALSE;
-	if (*s++ == '=') {
-	    s += strspn(s, " ");
-	    e = s;
-	    while (*e && !isspace(UCHAR(*e))) {
-		++e;
+    start = Ns_StrCaseFind(type, "charset");
+    if (start != NULL) {
+	start += 7;
+	start += strspn(start, " ");
+	if (*start++ == '=') {
+	    start += strspn(start, " ");
+	    end = start;
+	    while (*end && !isspace(UCHAR(*end))) {
+		++end;
 	    }
-	    ret_enc = GetCharsetEncoding(s, e-s);
-        }
-    } else if (strncasecmp(type, "text/", 5) == 0) {
-        *used_default = NS_TRUE;
-        ret_enc = GetDefaultEncoding();
-    } else {
-        *used_default = NS_FALSE;
+	    *lenPtr = end - start;
+	    return start;
+	}
     }
-
-    return ret_enc;
+    return NULL;
 }
+
 
 /*
  *----------------------------------------------------------------------
  *
- * NsComputeEncodingFromType --
+ * NsTclRegisterEncodingObjCmd --
  *
- *	Return the Tcl_Encoding for the given Content-type header,
- *	e.g., "text/html; charset=iso-8859-1" returns Tcl_Encoding
- *	for iso8859-1.  If the HackContentType configuration parameter
- *      is set to TRUE, the given content-type string will be updated
- *      with charset specification if it was not present.
+ *	Implements ns_register_encoding command to register an input
+ *	encoding for a given method/URL.
  *
  * Results:
- *	Tcl_Encoding or NULL if not found.  If a new type header is
- *      implied, then new_type is set to NS_TRUE, and the new header
- *      string is stored into type_ds.  When new_type is returned as
- *      NS_TRUE, it is the caller's responsibility to free the
- *      string in type_ds via Tcl_DStringFree.
+ *	Standard Tcl result. 
  *
  * Side effects:
- *	See LoadEncoding().
+ *	May register a new encoding for given method/URL.
  *
  *----------------------------------------------------------------------
  */
 
-void
-NsComputeEncodingFromType(char *type, Tcl_Encoding *enc,
-                          int *new_type, Tcl_DString *type_ds)
+int
+NsTclRegisterEncodingObjCmd(ClientData data, Tcl_Interp *interp, int objc,
+			   Tcl_Obj **objv)
 {
-    int          used_default;
+    NsInterp   *itPtr = data;
+    int         flags, idx;
+    Tcl_Encoding encoding;
+    char       *server, *method, *url, *charset;
 
-    *enc = NsGetTypeEncodingWithDef(type, &used_default);
-    if( used_default && GetDefaultHackContentTypeP() ) {
-        Tcl_DStringInit(type_ds);
-        Tcl_DStringAppend(type_ds, type, -1);
-        Tcl_DStringAppend(type_ds, "; charset=", -1);
-        Tcl_DStringAppend(type_ds, (char*)GetDefaultCharset(), -1);
-        *new_type = NS_TRUE;
-    } else {
-        *new_type = NS_FALSE;
+    if (objc != 4 && objc != 5) {
+badargs:
+        Tcl_WrongNumArgs(interp, 1, objv, "?-noinherit? method url charset");
+        return TCL_ERROR;
     }
+    if (STREQ(Tcl_GetString(objv[1]), "-noinherit")) {
+	if (objc < 5) {
+	    goto badargs;
+	}
+	flags = NS_OP_NOINHERIT;
+	idx = 2;
+    } else {
+	if (objc == 7) {
+	    goto badargs;
+	}
+	flags = 0;
+	idx = 1;
+    }
+    if (NsTclGetServer(itPtr, &server) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    method = Tcl_GetString(objv[idx++]);
+    url = Tcl_GetString(objv[idx++]);
+    charset = Tcl_GetString(objv[idx++]);
+    encoding = Ns_GetCharsetEncoding(charset);
+    if (encoding == NULL) {
+	Tcl_AppendResult(interp, "no encoding for charset: ", charset, NULL);
+	return TCL_ERROR;
+    }
+    Ns_UrlSpecificSet(server, method, url, eid, encoding, flags, NULL);
+    return TCL_OK;
 }
+
 
 /*
  *----------------------------------------------------------------------
@@ -529,13 +519,19 @@ int
 NsTclCharsetsCmd(ClientData dummy, Tcl_Interp *interp, int argc,
     char *argv[])
 {
+    Tcl_HashEntry  *entry;
+    Tcl_HashSearch  search;
 
-    Tcl_SetResult(interp, charsetList, TCL_VOLATILE);
-
+    Ns_MutexLock(&lock);
+    entry = Tcl_FirstHashEntry(&charsets, &search);
+    while (entry != NULL) {
+	Tcl_AppendElement(interp, (char *) Tcl_GetHashKey(&charsets, entry));
+	entry = Tcl_NextHashEntry(&search);
+    }
+    Ns_MutexUnlock(&lock);
     return TCL_OK;
 }
 
-
 
 /*
  *----------------------------------------------------------------------
@@ -573,13 +569,44 @@ NsTclEncodingForCharsetCmd(ClientData dummy, Tcl_Interp *interp, int argc,
     return TCL_OK;
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * NsGetInputEncoding --
+ *
+ *	Return the registered input encoding for the connection
+ *	method/url, if any.
+ *
+ * Results:
+ *	Pointer to Tcl_Encoding or NULL if no encoding registered.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Tcl_Encoding
+NsGetInputEncoding(Conn *connPtr)
+{
+    Tcl_Encoding encoding;
+
+    encoding = Ns_UrlSpecificGet(connPtr->server, connPtr->request->method,
+			     	 connPtr->request->url, eid);
+    if (encoding == NULL) {
+	encoding = connPtr->servPtr->inputEncoding;
+    }
+    return encoding;
+}
+
 
 /*
  *----------------------------------------------------------------------
  *
  * AddCharset, AddExtension --
  *
- *	Add 
+ *	Add charsets and extensions to hash tables.
  *
  * Results:
  *	None. 
@@ -614,182 +641,30 @@ AddCharset(char *charset, char *name)
     Ns_DStringFree(&ds);
 }
 
-
-/*
- *----------------------------------------------------------------------
- *
- * GetCharsetEncoding --
- *
- *	Cleanup charset name and return encoding, if any.
- *
- * Results:
- *	Tcl_Encoding or null if not known or not loaded.
- *
- * Side effects:
- *	None. 
- *
- *----------------------------------------------------------------------
- */
 
-static Tcl_Encoding
-GetCharsetEncoding(char *charset, int len)
+Tcl_Encoding
+NsGetOutputEncoding(Conn *connPtr)
 {
-    Tcl_HashEntry *hPtr;
     Tcl_Encoding encoding;
-    Ns_DString ds;
+    char *type, *charset;
+    int len;
 
     /*
-     * Cleanup the charset name and check for an
-     * alias (e.g., iso-8859-1 = iso8859-1) before
-     * assuming the charset and Tcl encoding names
-     * match (e.g., big5).
+     * Determine the output type based on the charset for text/
+     * types, using the server default charset if necessary.
      */
-    
-    Ns_DStringInit(&ds);
-    Ns_DStringNAppend(&ds, charset, len);
-    charset = Ns_StrTrim(Ns_StrToLower(ds.string));
-    hPtr = Tcl_FindHashEntry(&charsets, charset);
-    if (hPtr != NULL) {
-	charset = Tcl_GetHashValue(hPtr);
+
+   encoding = NULL;
+   type = Ns_GetMimeType(connPtr->request->url);
+    if (type != NULL && (strncmp(type, "text/", 5) == 0)) {
+    	charset = Ns_FindCharset(type, &len);
+    	if (charset == NULL) {
+	    charset = connPtr->servPtr->defcharset;
+	    len = -1;
+	}
+    	if (charset != NULL) {
+	    encoding = Ns_GetCharsetEncodingEx(charset, len);
+	}
     }
-    encoding = Ns_GetEncoding(charset);
-    Ns_DStringFree(&ds);
     return encoding;
 }
-
-
-/*
- *----------------------------------------------------------------------
- *
- * RebuildCharsetList --
- *
- *	Construct (or reconstruct) the list of charset names
- *      which are being kept precomputed and in sync with
- *      the current charset table.
- *
- * Results:
- *	char * string containing list of charset names
- *
- * Side effects:
- *	Retains the charset list string in static storage.
- *
- *----------------------------------------------------------------------
- */
-
-static char *
-RebuildCharsetList(void)
-{
-    Tcl_HashEntry  *entry;
-    Tcl_HashSearch  search;
-    Tcl_DString     ds;
-
-    Tcl_DStringInit(&ds);
-
-    Ns_MutexLock(&lock);
-    entry = Tcl_FirstHashEntry(&charsets, &search);
-    while (entry != NULL) {
-	Tcl_DStringAppendElement(&ds,
-	    (char *) Tcl_GetHashKey(&charsets, entry));
-	entry = Tcl_NextHashEntry(&search);
-    }
-    Ns_MutexUnlock(&lock);
-
-    if (charsetList != NULL ) {
-        ns_free( charsetList );
-    }
-
-    charsetList = ns_strdup( Tcl_DStringValue(&ds) );
-    Tcl_DStringFree(&ds);
-    return charsetList;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * GetDefaultEncoding --
- *
- *	Locate the appropriate default encoding value from within
- *      the current context.
- *
- * Results:
- *	Tcl_Encoding encoding reference
- *
- * Side effects:
- *	none.
- *
- *----------------------------------------------------------------------
- */
-
-static Tcl_Encoding
-GetDefaultEncoding(void)
-{
-    Conn         *connPtr;
-
-    if ( ((connPtr = (Conn *)Ns_GetConn()) != NULL) &&
-         (connPtr->servPtr != NULL) ) {
-        return connPtr->servPtr->encoding.outputEncoding;
-    } else {
-        return nsconf.encoding.outputEncoding;
-    }
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * GetDefaultCharset --
- *
- *	Locate the appropriate default output charset value from within
- *      the current context.
- *
- * Results:
- *	char * 
- *
- * Side effects:
- *	none.
- *
- *----------------------------------------------------------------------
- */
-
-static Tcl_Encoding
-GetDefaultCharset(void)
-{
-    Conn         *connPtr;
-
-    if ( ((connPtr = (Conn *)Ns_GetConn()) != NULL) &&
-         (connPtr->servPtr != NULL) ) {
-        return (Tcl_Encoding)connPtr->servPtr->encoding.outputCharset;
-    } else {
-        return (Tcl_Encoding)nsconf.encoding.outputCharset;
-    }
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * GetDefaultHackContentTypeP --
- *
- *	Locate the appropriate default encoding value from within
- *      the current context.
- *
- * Results:
- *	default HackContentTypeP flag.
- *
- * Side effects:
- *	none.
- *
- *----------------------------------------------------------------------
- */
-
-static int
-GetDefaultHackContentTypeP(void)
-{
-    Conn         *connPtr;
-
-    if ( ((connPtr = (Conn *)Ns_GetConn()) != NULL) &&
-         (connPtr->servPtr != NULL) ) {
-        return connPtr->servPtr->encoding.hackContentTypeP;
-    } else {
-        return nsconf.encoding.hackContentTypeP;
-    }
-}
