@@ -34,12 +34,9 @@
  *      Get page possibly from a file cache.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/fastpath.c,v 1.18.2.2 2004/08/17 19:46:23 dossy Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/fastpath.c,v 1.26 2006/04/19 17:48:47 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
-#ifndef _WIN32
-#include <sys/mman.h>
-#endif
 
 /*
  * The following constants are defined for this file
@@ -292,7 +289,7 @@ FastGetRestart(Ns_Conn *conn, char *page)
 
 /*
  *----------------------------------------------------------------------
- * NsFastGet --
+ * Ns_FastPathOp --
  *
  *      Return the contents of a URL.
  *
@@ -306,13 +303,14 @@ FastGetRestart(Ns_Conn *conn, char *page)
  */
 
 int
-NsFastGet(void *arg, Ns_Conn *conn)
+Ns_FastPathOp(void *arg, Ns_Conn *conn)
 {
-    Ns_DString      ds;
-    NsServer  	   *servPtr = arg;
+    Conn	   *connPtr = (Conn *) conn;
+    NsServer  	   *servPtr = connPtr->servPtr;
     char	   *url = conn->request->url;
-    int             result, i;
+    Ns_DString      ds;
     struct stat	    st;
+    int             result, i;
 
     Ns_DStringInit(&ds);
     if (NsUrlToFile(&ds, servPtr, url) != NS_OK
@@ -471,12 +469,12 @@ static int
 FastReturn(NsServer *servPtr, Ns_Conn *conn, int status,
     char *type, char *file, struct stat *stPtr)
 {
-    int         result = NS_ERROR, fd, new, nread;
+    int             result = NS_ERROR, fd, new, nread;
     File	   *filePtr;
     char	   *key;
     Ns_Entry	   *entPtr;
+    void           *map, *arg;
 #ifndef _WIN32
-    char           *map;
     FileKey	    ukey;
 #endif
 
@@ -521,19 +519,19 @@ FastReturn(NsServer *servPtr, Ns_Conn *conn, int status,
 		   file, strerror(errno));
 	    goto notfound;
 	}
-#ifndef _WIN32
 	if (servPtr->fastpath.mmap) {
-	    map = mmap(0, (size_t) stPtr->st_size, PROT_READ, MAP_SHARED, fd, 0);
-	    if (map != MAP_FAILED) {
+	    map = NsMap(fd, 0, stPtr->st_size, 0, &arg);
+	    if (map != NULL) {
 	    	close(fd);
 	    	fd = -1;
-            	result = Ns_ConnReturnData(conn, status, map, (int) stPtr->st_size, type);
-	    	munmap(map, (size_t) stPtr->st_size);
+            	result = Ns_ConnReturnData(conn, status, map,
+					   (int) stPtr->st_size, type);
+		NsUnMap(map, arg);
 	    }
 	}
-#endif
 	if (fd != -1) {
-            result = Ns_ConnReturnOpenFd(conn, status, type, fd, stPtr->st_size);
+            result = Ns_ConnReturnOpenFd(conn, status, type, fd,
+					 (int) stPtr->st_size);
 	    close(fd);
 	}
 
@@ -578,7 +576,7 @@ FastReturn(NsServer *servPtr, Ns_Conn *conn, int status,
 	        Ns_Log(Warning, "fastpath: failed to open '%s': '%s'",
 		       file, strerror(errno));
 	    } else {
-		filePtr = ns_malloc(sizeof(File) + stPtr->st_size);
+		filePtr = ns_malloc(sizeof(File) + (size_t) stPtr->st_size);
 		filePtr->refcnt = 1;
 		filePtr->size = stPtr->st_size;
 		filePtr->mtime = stPtr->st_mtime;
