@@ -33,7 +33,7 @@
  *	Ns_Info* API and ns_info command support.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/info.c,v 1.16 2003/06/02 14:51:33 vasiljevic Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/info.c,v 1.19 2005/08/23 21:41:31 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -494,6 +494,7 @@ int
 NsTclInfoObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj **objv)
 {
     NsInterp *itPtr = arg;
+    char *server;
     char *elog;
     Tcl_DString ds;
     static CONST char *opts[] = {
@@ -511,7 +512,7 @@ NsTclInfoObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj **objv)
 	IPidIdx, IPlatformIdx, IPoolsIdx, IScheduledIdx, IServerIdx, IServersIdx,
 	sockICallbacksIdx, ITagIdx, ITclLibIdx, IThreadsIdx, IUptimeIdx,
 	IVersionIdx, IWinntIdx,
-    } opt;
+    } _nsmayalias opt;
 
     if (objc != 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "option");
@@ -642,23 +643,24 @@ NsTclInfoObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj **objv)
 	break;
 
     case IServersIdx:
-	Tcl_SetResult(interp, nsconf.servers.string, TCL_STATIC);
+	Tcl_SetResult(interp, NsGetServers(), TCL_STATIC);
 	break;
 
+    case IServerIdx:
+	if (NsTclGetServer(itPtr, &server) != TCL_OK) {
+            return TCL_ERROR;
+        }
+        Tcl_SetResult(interp, itPtr->servPtr->server, TCL_STATIC);
+        break;
+
     case ITclLibIdx:
     case IPageRootIdx:
-    case IServerIdx:
-	if (itPtr->servPtr == NULL) {
-	    Tcl_SetResult(interp, "no server", TCL_STATIC);
-	    return TCL_ERROR;
-	}
-	if (opt == IServerIdx) {
-            Tcl_SetResult(interp, itPtr->servPtr->server, TCL_STATIC);
-	} else if (opt == ITclLibIdx) {
+	if (opt == ITclLibIdx) {
 	    Tcl_SetResult(interp, itPtr->servPtr->tcl.library, TCL_STATIC);
 	} else {
 	    Tcl_SetResult(interp, itPtr->servPtr->fastpath.pageroot, TCL_STATIC);
 	}
+        break;
     }
     return TCL_OK;
 }
