@@ -33,7 +33,7 @@
  *	AOLserver libnsd entry.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/init.c,v 1.5.2.1 2004/07/19 02:43:51 dossy Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/init.c,v 1.13 2005/10/08 20:20:51 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -41,10 +41,11 @@ static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/init.
 /*
  *----------------------------------------------------------------------
  *
- * NsdInit --
+ * Ns_LibInit --
  *
- *	Library entry point for libnsd.  This routine calls various
- *	data structure initialization functions throughout the core.
+ *	Initializes core nsd library.  Should be called automatically
+ *	during dynamic library initialization, otherwise explicitly for
+ *	static builds.
  *
  * Results:
  	None.
@@ -56,29 +57,54 @@ static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/init.
  */
 
 void
-NsdInit(void)
+Ns_LibInit(void)
 {
     static int once = 0;
 
     if (!once) {
 	once = 1;
+
+	/*
+	 * Initialize the threads interface.
+	 */
+
+	NsThreads_LibInit();
+
+	/*
+	 * Log must be initialized first in case later inits log messages.
+	 */
+
+    	NsInitLog();
+	NsInitFd();
+
+	/*
+	 * Caches and URL space are used by some of the remaining inits.
+	 */
+
+    	NsInitCache();
+    	NsInitUrlSpace();
+
+	/*
+	 * Order of the remaining inits is not significant.
+	 */
+
 #ifndef _WIN32
     	NsInitBinder();
 #endif
-    	NsInitCache();
     	NsInitConf();
+    	NsInitConfig();
+    	NsInitDrivers();
     	NsInitEncodings();
+        NsInitLimits();
     	NsInitListen();
-    	NsInitLog();
-    	NsInitInfo();
     	NsInitMimeTypes();
     	NsInitModLoad();
+        NsInitPools();
     	NsInitProcInfo();
-    	NsInitDrivers();
     	NsInitQueue();
-    	NsInitSched();
-    	NsInitTcl();
-    	NsInitUrlSpace();
     	NsInitRequests();
+    	NsInitSched();
+    	NsInitServers();
+    	NsInitTcl();
     }
 }
