@@ -28,7 +28,7 @@
 #
 
 #
-# $Header: /cvsroot/aolserver/aolserver/nsd/init.tcl,v 1.30.2.1 2004/07/02 04:11:02 dossy Exp $
+# $Header: /cvsroot/aolserver/aolserver/nsd/init.tcl,v 1.33 2006/04/13 19:06:28 jgdavidson Exp $
 #
 
 #
@@ -208,24 +208,13 @@ proc _ns_helper_eval {args} {
 
     set code [catch {uplevel 1 [eval concat $args]} result]
 
-    if $didsaveproc {
+    if {$didsaveproc} {
 	rename ns_eval ""
 	rename _saved_ns_eval ns_eval
     }
     return -code $code $result
 }
 
-#
-# ns_adp_include --
-#
-#   Wrapper for _ns_adp_include to ensure a
-#   new call frame with private variables.
-#
-
-proc ns_adp_include {args} {
-    eval _ns_adp_include $args
-}
-
 
 #
 # ns_init --
@@ -320,11 +309,10 @@ proc ns_cleanupvars {} {
 #
 
 proc ns_reinit {} {
-    ns_cleanup
-    ns_init
+    ns_ictl runtraces deallocate
+    ns_ictl runtraces allocate
 }
 
-
 #
 # _ns_savenamespaces --
 #
@@ -660,6 +648,14 @@ rename _ns_sourcefile   {}
 ns_cleanup
 _ns_savenamespaces
 
+#
+# Register the init and cleanup callbacks.
+#
+
+ns_ictl trace create {ns_ictl update}
+ns_ictl trace allocate ns_init
+ns_ictl trace deallocate ns_cleanup
+
 #
 # Kill this interp to save memory.
 #
