@@ -34,7 +34,7 @@
  *	Defines standard default mime types. 
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/mimetypes.c,v 1.11.2.1 2004/07/02 17:44:46 dossy Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/mimetypes.c,v 1.13 2005/09/27 02:26:27 shmooved Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 #define TYPE_DEFAULT "*/*"
@@ -83,75 +83,179 @@ static struct exttype {
     { ".aiff",  "audio/aiff" },
     { ".ani",   "application/x-navi-animation" },
     { ".art",   "image/x-art" },
+    { ".asc",   "text/plain" },
     { ".au",    "audio/basic" },
     { ".avi",   "video/x-msvideo" },
-    { ".bin",   "application/x-macbinary" },
+    { ".bcpio", "application/x-bcpio" },
+    { ".bin",   "application/octet-stream" },
     { ".bmp",   "image/bmp" },
+    { ".cdf",   "application/x-netcdf" },
+    { ".cgm",   "image/cgm" },
+    { ".class", "application/octet-stream" },
+    { ".cpio",  "application/x-cpio" },
+    { ".cpt",   "application/mac-compactpro" },
     { ".css",   "text/css" },
     { ".csv",   "application/csv" },
     { ".dcr",   "application/x-director" },
+    { ".der",   "application/x-x509-ca-cert" },
     { ".dir",   "application/x-director" },
+    { ".dll",   "application/octet-stream" },
+    { ".dms",   "application/octet-stream" },
+    { ".doc",   "application/msword" },
     { ".dp",    "application/commonground" },
+    { ".dvi",   "applications/x-dvi" },
+    { ".dwg",   "image/vnd.dwg" },
+    { ".dxf",   "image/vnd.dxf" },
     { ".dxr",   "application/x-director" },
     { ".elm",   "text/plain" },
     { ".eml",   "text/plain" },
+    { ".etx",   "text/x-setext" },
     { ".exe",   "application/octet-stream" },
+    { ".ez",    "application/andrew-inset" },
+    { ".fm",    "application/vnd.framemaker" },
     { ".gbt",   "text/plain" },    
     { ".gif",   "image/gif" },
-    { ".gz",    "application/x-compressed" },
+    { ".gtar",  "application/x-gtar" },
+    { ".gz",    "application/x-gzip" },
+    { ".hdf",   "application/x-hdf" },
+    { ".hpgl",  "application/vnd.hp-hpgl" },
     { ".hqx",   "application/mac-binhex40" },
+    { ".ice",   "x-conference/x-cooltalk" },
+    { ".ief",   "image/ief" },
+    { ".igs",   "image/iges" },
+    { ".iges",  "image/iges" },
     { ".jfif",  "image/jpeg" },
     { ".jpe",   "image/jpeg" },
     { ".jpg",   "image/jpeg" },
     { ".jpeg",  "image/jpeg" },
     { ".js",    "application/x-javascript" },
+    { ".kar",   "audio/midi" },
+    { ".latex", "application/x-latex" },
+    { ".lha",   "application/octet-stream" },
     { ".ls",    "application/x-javascript" },
+    { ".lxc",   "application/vnd.ms-excel" },
+    { ".lzh",   "application/octet-stream" },
+    { ".man",   "application/x-troff-man" },
     { ".map",   "application/x-navimap" },
+    { ".me",    "application/x-troff-me" },
+    { ".mesh",  "model/mesh" },
     { ".mid",   "audio/x-midi" },
     { ".midi",  "audio/x-midi" },
+    { ".mif",   "application/vnd.mif" },
     { ".mocha", "application/x-javascript" },
     { ".mov",   "video/quicktime" },
+    { ".movie", "video/x-sgi-movie" },
+    { ".mp2",   "audio/mpeg" },
+    { ".mp3",   "audio/mpeg" },
     { ".mpe",   "video/mpeg" },
     { ".mpeg",  "video/mpeg" },
     { ".mpg",   "video/mpeg" },
+    { ".mpga",  "audio/mpeg" },
+    { ".ms",    "application/x-troff-ms" },
+    { ".msh",   "model/mesh" },
+    { ".nc",    "application/x-netcdf" },
     { ".nvd",   "application/x-navidoc" },
     { ".nvm",   "application/x-navimap" },
+    { ".oda",   "application/oda" },
     { ".pbm",   "image/x-portable-bitmap" },
+    { ".pcl",   "application/vnd.hp-pcl" },
+    { ".pclx",  "application/vnd.hp-pclx" },
+    { ".pdb",   "chemical/x-pdb" },
     { ".pdf",   "application/pdf" },
     { ".pgm",   "image/x-portable-graymap" },
+    { ".pgn",   "application/x-chess-pgn" },
     { ".pic",   "image/pict" },
     { ".pict",  "image/pict" },
     { ".pnm",   "image/x-portable-anymap" },
     { ".png",   "image/png" },
+    { ".pot",   "application/vnd.ms-powerpoint" },
+    { ".ppm",   "image/x-portable-pixmap" },
+    { ".pps",   "application/vnd.ms-powerpoint" },
+    { ".ppt",   "application/vnd.ms-powerpoint" },
     { ".ps",    "application/postscript" },
     { ".qt",    "video/quicktime" },
-    { ".ra",    "audio/x-pn-realaudio" },
+    { ".ra",    "audio/x-realaudio" },
     { ".ram",   "audio/x-pn-realaudio" },
     { ".ras",   "image/x-cmu-raster" },
     { ".rgb",   "image/x-rgb" },
+    { ".rm",    "audio/x-pn-realaudio" },
+    { ".roff",  "application/x-troff" },
+    { ".rpm",   "audio/x-pn-realaudio-plugin" },
     { ".rtf",   "application/rtf" },
+    { ".rtx",   "text/richtext" },
+    { ".sda",   "application/vnd.stardivision.draw" },
+    { ".sdc",   "application/vnd.stardivision.calc" },
+    { ".sdd",   "application/vnd.stardivision.impress" },
+    { ".sdp",   "application/vnd.stardivision.impress" },
+    { ".sdw",   "application/vnd.stardivision.writer" },
+    { ".sgl",   "application/vnd.stardivision.writer-global" },
+    { ".sgm",   "text/sgml" },
+    { ".sgml",  "text/sgml" },
+    { ".sh",    "application/x-sh" },
+    { ".shar",  "application/x-shar" },
+    { ".silo",  "model/mesh" },
     { ".sit",   "application/x-stuffit" },
+    { ".skd",   "application/vnd.stardivision.math" },
+    { ".skm",   "application/vnd.stardivision.math" },
+    { ".skp",   "application/vnd.stardivision.math" },
+    { ".skt",   "application/vnd.stardivision.math" },
+    { ".smf",   "application/vnd.stardivision.math" },
+    { ".smi",   "application/smil" },
+    { ".smil",  "application/smil" },
     { ".snd",   "audio/basic" },
+    { ".spl",   "application/x-futuresplash" },
     { ".sql",   "application/x-sql" },
+    { ".src",   "application/x-wais-source" },
+    { ".stc",   "application/vnd.sun.xml.calc.template" },
+    { ".std",   "application/vnd.sun.xml.draw.template" },
+    { ".sti",   "application/vnd.sun.xml.impress.template" },
     { ".stl",   "application/x-navistyle" },
+    { ".stw",   "application/vnd.sun.xml.writer.template" },
+    { ".swf",   "application/x-shockwave-flash" },
+    { ".sxc",   "application/vnd.sun.xml.calc" },
+    { ".sxd",   "application/vnd.sun.xml.draw" },
+    { ".sxg",   "application/vnd.sun.xml.writer.global" },
+    { ".sxl",   "application/vnd.sun.xml.impress" },
+    { ".sxm",   "application/vnd.sun.xml.math" },
+    { ".sxw",   "application/vnd.sun.xml.writer" },
+    { ".t",     "application/x-troff" },
     { ".tar",   "application/x-tar" },
-    { ".tcl",   "text/plain" },
+    { ".tcl",   "x-tcl" },
+    { ".tex",   "application/x-tex" },
+    { ".texi",  "application/x-texinfo" },
+    { ".texinfo", "application/x-texinfo" },
     { ".text",  "text/plain" },
-    { ".tgz",   "application/x-compressed" },
+    { ".tgz",   "application/x-gtar" },
     { ".tif",   "image/tiff" },
     { ".tiff",  "image/tiff" },
+    { ".tr",    "application/x-troff" },
+    { ".tsv",   "text/tab-separated-values" },
     { ".txt",   "text/plain" },
-    { ".xbm",   "image/x-xbitmap" },
-    { ".xpm",   "image/x-xpixmap" },
-    { ".xht",   "application/xhtml+xml"},
-    { ".xhtml", "application/xhtml+xml"},
-    { ".xml",   "text/xml"},
-    { ".xsl",   "text/xml"},
-    { ".vrml",  "x-world/x-vrml" },
+    { ".ustar", "application/x-ustar" },
+    { ".vcd",   "application/x-cdlink" },
+    { ".vor",   "application/vnd.stardivision.writer" },
+    { ".vrml",  "model/vrml" },
     { ".wav",   "audio/x-wav" },
-    { ".wrl",   "x-world/x-vrml" },
-    { ".z",     "application/x-compressed" },
-    { ".zip",   "application/x-zip-compressed" },
+    { ".wbmp",  "image/vnd.wap.wbmp" },
+    { ".wkb",   "application/vnd.ms-excel" },
+    { ".wks",   "application/vnd.ms-excel" },
+    { ".wml",   "text/vnd.wap.wml" },
+    { ".wmlc",  "application/vnd.wap.wmlc" },
+    { ".wmls",  "text/vnd.wap.wmlscript" },
+    { ".wmlsc", "application/vnd.wap.wmlscript" },
+    { ".wrl",   "model/vrml" },
+    { ".xbm",   "image/x-xbitmap" },
+    { ".xls",   "application/vnd.ms-excel" },
+    { ".xlw",   "application/vnd.ms-excel" },
+    { ".xpm",   "image/x-xpixmap" },
+    { ".xht",   "application/xhtml+xml" },
+    { ".xhtml", "application/xhtml+xml" },
+    { ".xml",   "text/xml" },
+    { ".xsl",   "text/xml" },
+    { ".xyz",   "chemical/x-pdb" },
+    { ".xwd",   "image/x-xwindowdump" },
+    { ".z",     "application/x-compress" },
+    { ".zip",   "application/zip" },
     { NULL,     NULL }
 };
 
