@@ -117,7 +117,7 @@
 #define NSD_NAME             "AOLserver"
 #define NSD_VERSION	     NS_PATCH_LEVEL
 #define NSD_LABEL            "aolserver4_5"
-#define NSD_TAG              "$Name: aolserver_v45_r0 $"
+#define NSD_TAG              "$Name: aolserver_v45_r1 $"
 #define NS_CONFIG_PARAMETERS "ns/parameters"
 #define NS_CONFIG_THREADS    "ns/threads"
 
@@ -564,8 +564,11 @@ typedef struct Pool {
 	int 	    	    max;
     	int 	    	    current;
 	int 	    	    idle;
+	int 	    	    waiting;
+	int 	    	    starting;
 	int 	    	    timeout;
 	int		    maxconns;
+	int		    spread;
     	unsigned int	    queued;
     } threads;
 
@@ -983,7 +986,7 @@ extern void NsStartPools(void);
 extern void NsStopPools(Ns_Time *timeoutPtr);
 extern int NsTclGetPool(Tcl_Interp *interp, char *pool, Pool **poolPtrPtr);
 extern Tcl_ObjCmdProc NsTclListPoolsObjCmd;
-extern void NsCreateConnThread(Pool *poolPtr);
+extern void NsCreateConnThread(Pool *poolPtr, int joinThreads);
 extern void NsJoinConnThreads(void);
 extern int  NsStartDrivers(void);
 extern void NsWaitDriversShutdown(Ns_Time *toPtr);
