@@ -34,7 +34,7 @@
  *	Win32 specific routines.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/nswin32.c,v 1.11 2003/02/04 23:04:14 jrasmuss23 Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/nswin32.c,v 1.14 2005/08/11 23:00:44 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -59,8 +59,6 @@ static int sigpending;
 
 #define SysErrMsg()    (NsWin32ErrMsg(GetLastError()))
 
-void NsdInit(void);
-
 
 /*
  *----------------------------------------------------------------------
@@ -85,12 +83,13 @@ DllMain(HANDLE hModule, DWORD why, LPVOID lpReserved)
     WSADATA         wsd;
 
     if (why == DLL_PROCESS_ATTACH) {
+	NsThreads_LibInit();
 	Ns_TlsAlloc(&tls, ns_free);
 	if (WSAStartup(MAKEWORD(1, 1), &wsd) != 0) {
             return FALSE;
         }
         DisableThreadLibraryCalls(hModule);
-        NsdInit();
+	Ns_LibInit();
     } else if (why == DLL_PROCESS_DETACH) {
         WSACleanup();
     }
@@ -453,118 +452,6 @@ ns_sockdup(SOCKET sock)
     return (SOCKET) dup;
 }   
 
-
-/*
- *----------------------------------------------------------------------
- *
- * ns_pipe --
- *
- *	Create a pipe marked close-on-exec.
- *
- * Results:
- *  	0 if ok, -1 on error.
- *
- * Side effects:
- *  	None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-ns_pipe(int *fds)
-{
-    return _pipe(fds, 4096, _O_NOINHERIT|_O_BINARY);
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * ns_sockpair --
- *
- *	Create a pair of connected sockets via brute force.  Sock pairs
- *  	are used as trigger pipes in various subsystems.
- *
- * Results:
- *  	0 if ok, -1 on error.
- *
- * Side effects:
- *  	None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-ns_sockpair(SOCKET socks[2])
-{
-    SOCKET          sock;
-    struct sockaddr_in ia[2];
-    int             size;
-
-    size = sizeof(struct sockaddr_in);
-    sock = Ns_SockListen("127.0.0.1", 0);
-    if (sock == INVALID_SOCKET ||
-    	getsockname(sock, (struct sockaddr *) &ia[0], &size) != 0) {
-	return -1;
-    }
-    size = sizeof(struct sockaddr_in);
-    socks[1] = Ns_SockConnect("127.0.0.1", (int) ntohs(ia[0].sin_port));
-    if (socks[1] == INVALID_SOCKET ||
-    	getsockname(socks[1], (struct sockaddr *) &ia[1], &size) != 0) {
-	ns_sockclose(sock);
-	return -1;
-    }
-    size = sizeof(struct sockaddr_in);
-    socks[0] = accept(sock, (struct sockaddr *) &ia[0], &size);
-    ns_sockclose(sock);
-    if (socks[0] == INVALID_SOCKET) {
-	ns_sockclose(socks[1]);
-	return -1;
-    }
-    if (ia[0].sin_addr.s_addr != ia[1].sin_addr.s_addr ||
-	ia[0].sin_port != ia[1].sin_port) {
-	ns_sockclose(socks[0]);
-	ns_sockclose(socks[1]);
-	return -1;
-    }
-    return 0;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * Ns_SockListen --
- *
- *	Simple socket listen implementation for Win32 without privileged
- *  	port issues.
- *
- * Results:
- *	Socket descriptor or INVALID_SOCKET on error.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-SOCKET
-Ns_SockListenEx(char *address, int port, int backlog)
-{
-    SOCKET          sock;
-    struct sockaddr_in sa;
-
-    if (Ns_GetSockAddr(&sa, address, port) != NS_OK) {
-	return -1;
-    }
-    sock = Ns_SockBind(&sa);
-    if (sock != INVALID_SOCKET && listen(sock, backlog) != 0) {
-	ns_sockclose(sock);
-	sock = INVALID_SOCKET;
-    }
-    return sock;
-}
-
 
 /*
  *----------------------------------------------------------------------
@@ -866,84 +753,3 @@ ReportStatus(DWORD state, DWORD code, DWORD hint)
         Ns_Fatal("nswin32: SetServiceStatus(%d) failed: '%s'", state, SysErrMsg());
     }
 }
-
-
-/* Copyright (C) 1994, 1996, 1997 Free Software Foundation, Inc.
-   This file is part of the GNU C Library.
-
-   The GNU C Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   The GNU C Library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with the GNU C Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
-
-/* Poll the file descriptors described by the NFDS structures starting at
-   FDS.  If TIMEOUT is nonzero and not -1, allow TIMEOUT milliseconds for
-   an event to occur; if TIMEOUT is -1, block until an event occurs.
-   Returns the number of file descriptors with events, zero if timed out,
-   or -1 for errors.  */
-
-int
-poll (fds, nfds, timeout)
-     struct pollfd *fds;
-     unsigned long int nfds;
-     int timeout;
-{
- struct timeval tv, *tvp;
- fd_set rset, wset, xset;
- struct pollfd *f;
- int ready;
- int maxfd = 0;
- 
- FD_ZERO (&rset);
- FD_ZERO (&wset);
- FD_ZERO (&xset);
- 
- for (f = fds; f < &fds[nfds]; ++f)
-   if (f->fd != -1)
-   {
-    if (f->events & POLLIN)
-      FD_SET (f->fd, &rset);
-    if (f->events & POLLOUT)
-      FD_SET (f->fd, &wset);
-    if (f->events & POLLPRI)
-      FD_SET (f->fd, &xset);
-    if (f->fd > maxfd && (f->events & (POLLIN|POLLOUT|POLLPRI)))
-      maxfd = f->fd;
-   }
- 
- if (timeout < 0) {
-    tvp = NULL;
- } else {
-    tv.tv_sec = timeout / 1000;
-    tv.tv_usec = (timeout % 1000) * 1000;
-    tvp = &tv;
- }
- 
- ready = select (maxfd + 1, &rset, &wset, &xset, tvp);
- if (ready > 0)
-   for (f = fds; f < &fds[nfds]; ++f)
-   {
-    f->revents = 0;
-    if (f->fd >= 0)
-    {
-     if (FD_ISSET (f->fd, &rset))
-       f->revents |= POLLIN;
-     if (FD_ISSET (f->fd, &wset))
-       f->revents |= POLLOUT;
-     if (FD_ISSET (f->fd, &xset))
-       f->revents |= POLLPRI;
-    }
-   }
- 
- return ready;
-}
\ No newline at end of file
