@@ -34,10 +34,13 @@
  *  	routines (previously known as "op procs").
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/op.c,v 1.11.2.1 2004/07/14 00:36:52 dossy Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/op.c,v 1.15 2005/10/07 00:48:23 dossy Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
+#define MAX_RECURSION 3       /* Max return direct recursion limit. */
+
+
 /*
  * The following structure defines a request procedure including user
  * routine and client data.
@@ -214,19 +217,35 @@ int
 Ns_ConnRunRequest(Ns_Conn *conn)
 {
     Req *reqPtr;
+    Conn *connPtr = (Conn *) conn;
     int  status;
     char *server = Ns_ConnServer(conn);
 
+    /*
+     * Return a quick unavailable error on overflow.
+     */
+
+    if (connPtr->flags & NS_CONN_OVERFLOW) {
+        return Ns_ConnReturnServiceUnavailable(conn);
+    }
+
+    /*
+     * Prevent infinite internal redirect loops.
+     */
+
+    if (connPtr->recursionCount > MAX_RECURSION) {
+        Ns_Log(Error, "return: failed to redirect '%s %s': "
+               "exceeded recursion limit of %d",
+               conn->request->method, conn->request->url, MAX_RECURSION);
+        return Ns_ConnReturnInternalError(conn);
+    }
+
     Ns_MutexLock(&ulock);
     reqPtr = Ns_UrlSpecificGet(server, conn->request->method,
     	    	    	       conn->request->url, uid);
     if (reqPtr == NULL) {
     	Ns_MutexUnlock(&ulock);
-	if (STREQ(conn->request->method, "BAD")) {
-	    return Ns_ConnReturnBadRequest(conn, NULL);
-	} else {
-	   return Ns_ConnReturnNotFound(conn);
-	}
+        return Ns_ConnReturnNotFound(conn);
     }
     ++reqPtr->refcnt;
     Ns_MutexUnlock(&ulock);
@@ -261,8 +280,11 @@ Ns_ConnRunRequest(Ns_Conn *conn)
 int
 Ns_ConnRedirect(Ns_Conn *conn, char *url)
 {
+    Conn *connPtr = (Conn *) conn;
     int status;
 
+    ++connPtr->recursionCount;
+
     /*
      * Update the request URL.
      */
