@@ -34,7 +34,7 @@
  *	and service threads.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/queue.c,v 1.38 2006/04/13 19:06:24 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/queue.c,v 1.46 2008/12/27 00:36:38 gneumann Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -178,18 +178,37 @@ NsQueueConn(Conn *connPtr)
     }
     poolPtr->queue.wait.lastPtr = connPtr;
     connPtr->nextPtr = NULL;
-    if (poolPtr->threads.idle == 0
-            && poolPtr->threads.current < poolPtr->threads.max) {
-        ++poolPtr->threads.idle;
-        ++poolPtr->threads.current;
+
+    if (poolPtr->threads.waiting == 0
+        && poolPtr->threads.current < poolPtr->threads.max) {
+        /* 
+           Create a new thread if no thread is waiting and the number
+           of currently starting or running threads is below max.
+        */
         create = 1;
     }
-    ++poolPtr->queue.wait.num;
-    Ns_MutexUnlock(&poolPtr->lock);
+
+    poolPtr->queue.wait.num ++;
+    
     if (create) {
-        NsCreateConnThread(poolPtr);
-    } else {
+        poolPtr->threads.current ++;
+        Ns_MutexUnlock(&poolPtr->lock);
+        NsCreateConnThread(poolPtr, 1);
+    } else if (poolPtr->threads.waiting > 0) {
+        /*
+          There are threads waiting. Signal to process
+          the request.
+         */
         Ns_CondSignal(&poolPtr->cond);
+        Ns_MutexUnlock(&poolPtr->lock);
+    } else {
+        /* 
+           We might have missed signaling, since we are already using
+           max resources, and no thread is available. In such a case
+           the autorecovery at thread exist has to care to process
+           the outstanding requests 
+        */
+        Ns_MutexUnlock(&poolPtr->lock);
     }
 }
 
@@ -351,6 +370,7 @@ NsConnThread(void *arg)
     char             name[100];
     int              status, ncons;
     char            *msg;
+    double           spread;
     
     /*
      * Set the conn thread name.
@@ -358,10 +378,15 @@ NsConnThread(void *arg)
 
     Ns_TlsSet(&ctdtls, dataPtr);
     Ns_MutexLock(&poolPtr->lock);
-    sprintf(name, "-conn:%d-", poolPtr->threads.nextid++);
+    sprintf(name, "-%s:%d-", poolPtr->name, poolPtr->threads.nextid++);
     Ns_MutexUnlock(&poolPtr->lock);
     Ns_ThreadSetName(name);
-    ncons = poolPtr->threads.maxconns;
+
+    /* spread is a value of 1.0 +- specified percentage, 
+       i.e. between 0.0 and 2.0 when the configured percentage is 100 */
+    spread = 1.0 + (2 * poolPtr->threads.spread * Ns_DRand() - poolPtr->threads.spread) / 100.0;
+
+    ncons = round(poolPtr->threads.maxconns * spread);
     msg = "exceeded max connections per thread";
     
     /*
@@ -369,27 +394,37 @@ NsConnThread(void *arg)
      */
 
     Ns_MutexLock(&poolPtr->lock);
+
+    poolPtr->threads.starting--;
+    poolPtr->threads.idle++;
+
     while (poolPtr->threads.maxconns <= 0 || ncons-- > 0) {
 
 	/*
 	 * Wait for a connection to arrive, exiting if one doesn't
 	 * arrive in the configured timeout period.
 	 */
-
+        
 	if (poolPtr->threads.current <= poolPtr->threads.min) {
 	    timePtr = NULL;
 	} else {
 	    Ns_GetTime(&wait);
-	    Ns_IncrTime(&wait, poolPtr->threads.timeout, 0);
+	    Ns_IncrTime(&wait, round(poolPtr->threads.timeout * spread), 0);
 	    timePtr = &wait;
 	}
 
-	status = NS_OK;
+        status = NS_OK;
         while (!poolPtr->shutdown
-		&& status == NS_OK
-                && poolPtr->queue.wait.firstPtr == NULL) {
+               && status == NS_OK
+               && poolPtr->queue.wait.firstPtr == NULL) {
+            /* 
+               nothing is queued, we wait for a queue entry 
+            */
+            poolPtr->threads.waiting++;
             status = Ns_CondTimedWait(&poolPtr->cond, &poolPtr->lock, timePtr);
-	}
+            poolPtr->threads.waiting--;
+        }
+
 	if (poolPtr->queue.wait.firstPtr == NULL) {
 	    msg = "timeout waiting for connection";
 	    break;
@@ -406,76 +441,97 @@ NsConnThread(void *arg)
     	}
 	connPtr->nextPtr = NULL;
 	connPtr->prevPtr = poolPtr->queue.active.lastPtr;
-	if (poolPtr->queue.active.lastPtr != NULL) {
-	    poolPtr->queue.active.lastPtr->nextPtr = connPtr;
-	}
-	poolPtr->queue.active.lastPtr = connPtr;
-	if (poolPtr->queue.active.firstPtr == NULL) {
-	    poolPtr->queue.active.firstPtr = connPtr;
-	}
-	poolPtr->threads.idle--;
-	poolPtr->queue.wait.num--;
-        Ns_MutexUnlock(&poolPtr->lock);
+         if (poolPtr->queue.active.lastPtr != NULL) {
+             poolPtr->queue.active.lastPtr->nextPtr = connPtr;
+         }
+         poolPtr->queue.active.lastPtr = connPtr;
+         if (poolPtr->queue.active.firstPtr == NULL) {
+             poolPtr->queue.active.firstPtr = connPtr;
+         }
+         poolPtr->threads.idle--;
+         poolPtr->queue.wait.num--;
 
-	/*
-	 * Run the connection.
-	 */
+         Ns_MutexUnlock(&poolPtr->lock);
 
-	Ns_MutexLock(&connlock);
-        dataPtr->connPtr = connPtr;
-	Ns_MutexUnlock(&connlock);
+         /*
+          * Run the connection.
+          */
 
-	Ns_GetTime(&connPtr->times.run);
-        ConnRun(connPtr);
+         Ns_MutexLock(&connlock);
+         dataPtr->connPtr = connPtr;
+         Ns_MutexUnlock(&connlock);
+         
+         Ns_GetTime(&connPtr->times.run);
+         ConnRun(connPtr);
+         Ns_MutexLock(&connlock);
+         dataPtr->connPtr = NULL;
+         Ns_MutexUnlock(&connlock);
+         
+         /*
+          * Remove from the active list and push on the free list.
+          */
 
-	Ns_MutexLock(&connlock);
-        dataPtr->connPtr = NULL;
-	Ns_MutexUnlock(&connlock);
-
-	/*
-	 * Remove from the active list and push on the free list.
-	 */
-
-        Ns_MutexLock(&poolPtr->lock);
-	if (connPtr->prevPtr != NULL) {
-	    connPtr->prevPtr->nextPtr = connPtr->nextPtr;
-	} else {
-	    poolPtr->queue.active.firstPtr = connPtr->nextPtr;
-	}
-	if (connPtr->nextPtr != NULL) {
-	    connPtr->nextPtr->prevPtr = connPtr->prevPtr;
-	} else {
-	    poolPtr->queue.active.lastPtr = connPtr->prevPtr;
-	}
-	poolPtr->threads.idle++;
-        Ns_MutexUnlock(&poolPtr->lock);
-        NsFreeConn(connPtr);
-        Ns_MutexLock(&poolPtr->lock);
+         Ns_MutexLock(&poolPtr->lock);
+         if (connPtr->prevPtr != NULL) {
+             connPtr->prevPtr->nextPtr = connPtr->nextPtr;
+         } else {
+             poolPtr->queue.active.firstPtr = connPtr->nextPtr;
+         }
+         if (connPtr->nextPtr != NULL) {
+             connPtr->nextPtr->prevPtr = connPtr->prevPtr;
+         } else {
+             poolPtr->queue.active.lastPtr = connPtr->prevPtr;
+         }
+         poolPtr->threads.idle++;
+         Ns_MutexUnlock(&poolPtr->lock);
+         NsFreeConn(connPtr);
+         Ns_MutexLock(&poolPtr->lock);
     }
-
+    
     /*
      * Append this thread to list of threads to reap.
      */
-
+    
     Ns_MutexLock(&joinlock);
     dataPtr->nextPtr = joinPtr;
     joinPtr = dataPtr;
     Ns_MutexUnlock(&joinlock);
-
+    
     /*
      * Mark this thread as no longer active.
      */
-
+    
     if (poolPtr->shutdown) {
-	msg = "shutdown pending";
+        msg = "shutdown pending";
     }
-    poolPtr->threads.idle--;
     poolPtr->threads.current--;
-    if (poolPtr->threads.current == 0) {
-        Ns_CondBroadcast(&poolPtr->cond);
+    poolPtr->threads.idle--;
+    
+    if (((poolPtr->queue.wait.num > 0 
+          && poolPtr->threads.idle == 0 
+          && poolPtr->threads.starting == 0
+          )
+         || (poolPtr->threads.current < poolPtr->threads.min)
+         ) && !poolPtr->shutdown) {
+        /* 
+           Recreate a thread when on of the condings hold
+           - there are more queue entries are still waiting, 
+           but no thread is either starting or idle, or
+           - there are less than minthreads connection threads alive.
+        */
+        poolPtr->threads.current ++;
+        Ns_MutexUnlock(&poolPtr->lock);
+        NsCreateConnThread(poolPtr, 0); /* joinThreads == 0 to avoid deadlock */
+    } else if (poolPtr->queue.wait.num > 0 && poolPtr->threads.waiting > 0) {
+        /*
+          Wake up a waiting thread
+        */
+        Ns_CondSignal(&poolPtr->cond);
+        Ns_MutexUnlock(&poolPtr->lock);
+    } else {
+        Ns_MutexUnlock(&poolPtr->lock);
     }
-    Ns_MutexUnlock(&poolPtr->lock);
-
+    
     Ns_Log(Notice, "exiting: %s", msg);
     Ns_ThreadExit(dataPtr);
 }
@@ -619,7 +675,7 @@ ConnRun(Conn *connPtr)
  */
 
 void
-NsCreateConnThread(Pool *poolPtr)
+NsCreateConnThread(Pool *poolPtr, int joinThreads)
 {
     ConnData *dataPtr;
 
@@ -627,7 +683,9 @@ NsCreateConnThread(Pool *poolPtr)
      * Reap any dead threads.
      */
 
-    NsJoinConnThreads();
+    if (joinThreads) {
+        NsJoinConnThreads();
+    }
 
     /*
      * Create a new connection thread.
@@ -636,6 +694,9 @@ NsCreateConnThread(Pool *poolPtr)
     dataPtr = ns_malloc(sizeof(ConnData));
     dataPtr->poolPtr = poolPtr;
     dataPtr->connPtr = NULL;
+    Ns_MutexLock(&poolPtr->lock);
+    poolPtr->threads.starting ++;
+    Ns_MutexUnlock(&poolPtr->lock);
     Ns_ThreadCreate(NsConnThread, dataPtr, 0, &dataPtr->thread);
 }
  
