@@ -34,7 +34,7 @@
  *	Take text and make it safe for HTML. 
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/quotehtml.c,v 1.3 2000/08/02 23:38:25 kriston Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/quotehtml.c,v 1.5 2005/07/18 23:33:23 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -58,7 +58,11 @@ static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/quote
 void
 Ns_QuoteHtml(Ns_DString *pds, char *string)
 {
-    while (*string != '\0') {
+    char *end, *next;
+
+    end = string + strlen(string);
+    do {
+	next = Tcl_UtfNext(string);
         switch (*string) {
         case '<':
             Ns_DStringAppend(pds, "&lt;");
@@ -81,11 +85,11 @@ Ns_QuoteHtml(Ns_DString *pds, char *string)
 	    break;
 	    
 	default:
-            Ns_DStringNAppend(pds, string, 1);
+            Ns_DStringNAppend(pds, string, next - string);
             break;
         }
-        ++string;
-    }
+	string = next;
+    } while (string < end);
 }
 
 
@@ -117,6 +121,6 @@ NsTclQuoteHtmlCmd(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
     }
     Ns_DStringInit(&ds);
     Ns_QuoteHtml(&ds, argv[1]);
-    Tcl_SetResult(interp, Ns_DStringExport(&ds), (Tcl_FreeProc *) ns_free);
+    Tcl_DStringResult(interp, &ds);
     return TCL_OK;
 }
