@@ -33,7 +33,7 @@
  *	Routines for managing NsServer structures.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/server.c,v 1.45 2006/04/13 19:06:32 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/server.c,v 1.47 2008/12/05 08:51:43 gneumann Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -44,6 +44,7 @@ static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/serve
 static void GetCharsetEncoding(char *path, char *key, char **charsetPtr,
 		   	       Tcl_Encoding *encodingPtr);
 static NsServer *CreateServer(char *server);
+static void RegisterRedirects(char *server);
 static void RegisterMaps(char *server, char *type, Ns_OpProc *proc);
 static void RegisterMap(char *server, char *type, char *map, Ns_OpProc *proc);
 
@@ -202,6 +203,7 @@ NsInitServer(char *server, Ns_ServerInitProc *initProc)
     RegisterMap(server, "fastpath", "/", Ns_FastPathOp);
     RegisterMaps(server, "fastpath", Ns_FastPathOp);
     RegisterMaps(server, "adp", NsAdpProc);
+    RegisterRedirects(server);
 
     /*
      * Call the given init proc, if any, which may register
@@ -241,9 +243,9 @@ CreateServer(char *server)
     Tcl_Encoding outputEncoding;
     Ns_DString ds;
     NsServer *servPtr;
-    char *path, *spath, *map, *key, *dirf, *p;
+    char *path, *spath, *dirf, *p;
     Ns_Set *set;
-    int i, n, status;
+    int i, n;
 
     Ns_DStringInit(&ds);
     servPtr = ns_calloc(1, sizeof(NsServer));
@@ -334,10 +336,12 @@ CreateServer(char *server)
      */
      
     Ns_MutexInit(&servPtr->tcl.llock);
+    Ns_MutexSetName(&servPtr->tcl.llock, "ns:tcl.llock");
     Ns_CondInit(&servPtr->tcl.lcond);
     Ns_RWLockInit(&servPtr->tcl.tlock);
     Ns_CsInit(&servPtr->tcl.olock);
     Ns_MutexInit(&servPtr->tcl.plock);
+    Ns_MutexSetName(&servPtr->tcl.plock, "ns:tcl.plock");
     Ns_RWLockInit(&servPtr->tcl.slock);
     Tcl_InitHashTable(&servPtr->tcl.packages, TCL_STRING_KEYS);
     Tcl_InitHashTable(&servPtr->tcl.once, TCL_STRING_KEYS);
@@ -441,19 +445,7 @@ CreateServer(char *server)
     Tcl_InitHashTable(&servPtr->request.proxy, TCL_STRING_KEYS);
     Ns_MutexInit(&servPtr->request.plock);
     Ns_MutexSetName2(&servPtr->request.plock, "nsd:proxy", server);
-    path = Ns_ConfigGetPath(server, NULL, "redirects", NULL);
-    set = Ns_ConfigGetSection(path);
     Tcl_InitHashTable(&servPtr->request.redirect, TCL_ONE_WORD_KEYS);
-    for (i = 0; set != NULL && i < Ns_SetSize(set); ++i) {
-	key = Ns_SetKey(set, i);
-	map = Ns_SetValue(set, i);
-	status = atoi(key);
-	if (status <= 0 || *map == '\0') {
-	    Ns_Log(Error, "return: invalid redirect '%s=%s'", key, map);
-	} else {
-	    Ns_RegisterReturn(status, map);
-	}
-    }
 
     /*
      * Initialize ADP.
@@ -663,3 +655,41 @@ RegisterMap(char *server, char *type, char *map, Ns_OpProc *proc)
 	Tcl_Free((char *) largv);
     }
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * RegisterRedirects --
+ *
+ *	Register redirects for given server.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	See Ns_RegisterRedirect.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+RegisterRedirects(char *server)
+{
+    char *path, *key, *url;
+    Ns_Set *set;
+    int i, status;
+
+    path = Ns_ConfigGetPath(server, NULL, "redirects", NULL);
+    set = Ns_ConfigGetSection(path);
+    for (i = 0; set != NULL && i < Ns_SetSize(set); ++i) {
+	key = Ns_SetKey(set, i);
+	url = Ns_SetValue(set, i);
+	status = atoi(key);
+	if (status <= 0 || *url == '\0') {
+	    Ns_Log(Error, "return: invalid redirect '%s=%s'", key, url);
+	} else {
+	    Ns_RegisterRedirect(server, status, url);
+	}
+    }
+}
