@@ -34,7 +34,7 @@
  *	Wrappers and convenience functions for TCP/IP stuff. 
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/sock.c,v 1.11 2003/02/04 23:10:49 jrasmuss23 Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/sock.c,v 1.15 2005/10/09 16:16:54 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -130,45 +130,48 @@ Ns_SockSend(SOCKET sock, void *buf, int towrite, int timeout)
  */
 
 int
-Ns_SockWait(SOCKET sock, int what, int timeout)
+Ns_SockWait(SOCKET sock, int what, int seconds)
 {
+    return Ns_SockWaitEx(sock, what, seconds * 1000);
+}
+
+int
+Ns_SockWaitEx(SOCKET sock, int what, int ms)
+{
+    Ns_Time timeout;
     struct pollfd pfd;
     int n;
 
-    if (timeout < 0) {
-    	return NS_TIMEOUT;
+    if (ms < 0) {
+	n = 0;
+    } else {
+    	Ns_GetTime(&timeout);
+    	Ns_IncrTime(&timeout, 0, ms * 1000);
+    	pfd.fd = sock;
+    	switch (what) {
+    	case NS_SOCK_READ:
+	    pfd.events = POLLIN;
+	    break;
+    	case NS_SOCK_WRITE:
+	    pfd.events = POLLOUT;
+	    break;
+    	case NS_SOCK_EXCEPTION:
+	    pfd.events = POLLPRI;
+	    break;
+    	default:
+	    return NS_ERROR;
+	    break;
+    	}
+    	n = NsPoll(&pfd, 1, &timeout);
     }
-    timeout *= 1000;
-    pfd.fd = sock;
-    switch (what) {
-    case NS_SOCK_READ:
-	pfd.events = POLLIN;
-	break;
-    case NS_SOCK_WRITE:
-	pfd.events = POLLOUT;
-	break;
-    case NS_SOCK_EXCEPTION:
-	pfd.events = POLLPRI;
-	break;
-    default:
-	return NS_ERROR;
-	break;
-    }
-    pfd.revents = 0;
-    do {
-	n = poll(&pfd, 1, timeout);
-    } while (n < 0 && errno == EINTR);
-    if (n > 0) {
-	return NS_OK;
-    }
-    return NS_TIMEOUT;
+    return (n ? NS_OK : NS_TIMEOUT);
 }
 
 
 /*
  *----------------------------------------------------------------------
  *
- * Ns_SockListen --
+ * Ns_SockListen, Ns_SockListenEx --
  *
  *	Listen for connections with default backlog.
  *
@@ -187,6 +190,30 @@ Ns_SockListen(char *address, int port)
     return Ns_SockListenEx(address, port, nsconf.backlog);
 }
 
+SOCKET
+Ns_SockListenEx(char *address, int port, int backlog)
+{
+    SOCKET sock;
+    struct sockaddr_in sa;
+
+    if (Ns_GetSockAddr(&sa, address, port) != NS_OK) {
+	return -1;
+    }
+#ifdef WIN32
+    sock = Ns_SockBind(&sa);
+#else
+    sock = NsSockGetBound(&sa);
+    if (sock == INVALID_SOCKET) {
+	sock = Ns_SockBind(&sa);
+    }
+#endif
+    if (sock != INVALID_SOCKET && listen(sock, backlog) != 0) {
+	ns_sockclose(sock);
+	sock = INVALID_SOCKET;
+    }
+    return sock;
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -671,3 +698,169 @@ SockSetup(SOCKET sock)
 #endif
     return sock;
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * ns_poll --
+ *
+ *	Poll file descriptors.  Emulation using select(2) is provided
+ *	for platforms without poll(2).
+ *
+ * Results:
+ *	See poll(2) man page.
+ *
+ * Side effects:
+ *	See poll(2) man page.
+ *
+ *----------------------------------------------------------------------
+ */
+
+/* Copyright (C) 1994, 1996, 1997 Free Software Foundation, Inc.
+   This file is part of the GNU C Library.
+
+   The GNU C Library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public License as
+   published by the Free Software Foundation; either version 2 of the
+   License, or (at your option) any later version.
+
+   The GNU C Library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public
+   License along with the GNU C Library; see the file COPYING.LIB.  If not,
+   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
+
+/* Poll the file descriptors described by the NFDS structures starting at
+   FDS.  If TIMEOUT is nonzero and not -1, allow TIMEOUT milliseconds for
+   an event to occur; if TIMEOUT is -1, block until an event occurs.
+   Returns the number of file descriptors with events, zero if timed out,
+   or -1 for errors.  */
+
+int
+ns_poll(fds, nfds, timeout)
+     struct pollfd *fds;
+     unsigned long int nfds;
+     int timeout;
+{
+#ifdef HAVE_POLL
+ return poll(fds, nfds, timeout);
+#else
+ struct timeval tv, *tvp;
+ fd_set rset, wset, xset;
+ struct pollfd *f;
+ int ready;
+ int maxfd = 0;
+ 
+ FD_ZERO (&rset);
+ FD_ZERO (&wset);
+ FD_ZERO (&xset);
+ 
+ for (f = fds; f < &fds[nfds]; ++f)
+   if (f->fd != -1)
+   {
+    if (f->events & POLLIN)
+      FD_SET (f->fd, &rset);
+    if (f->events & POLLOUT)
+      FD_SET (f->fd, &wset);
+    if (f->events & POLLPRI)
+      FD_SET (f->fd, &xset);
+    if (f->fd > maxfd && (f->events & (POLLIN|POLLOUT|POLLPRI)))
+      maxfd = f->fd;
+   }
+ 
+ if (timeout < 0) {
+    tvp = NULL;
+ } else {
+    tv.tv_sec = timeout / 1000;
+    tv.tv_usec = (timeout % 1000) * 1000;
+    tvp = &tv;
+ }
+ 
+ ready = select (maxfd + 1, &rset, &wset, &xset, tvp);
+ if (ready > 0)
+   for (f = fds; f < &fds[nfds]; ++f)
+   {
+    f->revents = 0;
+    if (f->fd >= 0)
+    {
+     if (FD_ISSET (f->fd, &rset))
+       f->revents |= POLLIN;
+     if (FD_ISSET (f->fd, &wset))
+       f->revents |= POLLOUT;
+     if (FD_ISSET (f->fd, &xset))
+       f->revents |= POLLPRI;
+    }
+   }
+ 
+ return ready;
+#endif
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * NsPoll --
+ *
+ *	Poll file descriptors using an absolute timeout and restarting
+ *	after any interrupts which may be received.
+ *
+ * Results:
+ *	See poll(2) man page.
+ *
+ * Side effects:
+ *	See poll(2) man page.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+NsPoll(struct pollfd *pfds, int nfds, Ns_Time *timeoutPtr)
+{
+    Ns_Time now, diff;
+    int i, n, ms;
+
+    /*
+     * Clear revents.
+     */
+
+    for (i = 0; i < nfds; ++i) {
+	pfds[i].revents = 0;
+    }
+
+    /*
+     * Determine relative time from absolute time and continue polling
+     * if any interrupts are received.
+     */
+
+    do {
+	if (timeoutPtr == NULL) {
+            ms = -1;
+	} else {
+	    Ns_GetTime(&now);
+            if (Ns_DiffTime(timeoutPtr, &now, &diff) <= 0)  {
+                ms = 0;
+            } else {
+            	ms = diff.sec * 1000 + diff.usec / 1000;
+            }
+	}
+	n = ns_poll(pfds, (size_t) nfds, ms);
+    } while (n < 0 && ns_sockerrno == EINTR);
+
+    /*
+     * Poll errors are not tolerated in AOLserver as they must indicate
+     * a code error which if ignored could lead to data lose and/or
+     * endless polling loops and error messages.
+     */
+
+    if (n < 0) {
+	Ns_Fatal("poll() failed: %s", ns_sockstrerror(ns_sockerrno));
+    }
+
+    return n;
+}
