@@ -34,7 +34,7 @@
  *	Functions that deal with strings. 
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/str.c,v 1.5.2.1 2004/06/03 18:03:38 rcrittenden0569 Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/str.c,v 1.6 2004/06/08 19:28:57 rcrittenden0569 Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -84,8 +84,9 @@ Ns_StrTrim(char *string)
 char *
 Ns_StrTrimLeft(char *string)
 {
-    if (string == NULL)
-        return NULL;
+    if (string == NULL) {
+	return NULL;
+    }
     while (isspace(UCHAR(*string))) {
         ++string;
     }
@@ -116,8 +117,9 @@ Ns_StrTrimRight(char *string)
 {
     int len;
 
-    if (string == NULL)
-        return NULL;
+    if (string == NULL) {
+	return NULL;
+    }
     len = strlen(string);
     while ((--len >= 0) &&
 	   (isspace(UCHAR(string[len])) ||
