@@ -34,7 +34,7 @@
  *	Tcl commands that do stuff to the filesystem. 
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tclfile.c,v 1.24 2005/08/23 21:41:31 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tclfile.c,v 1.27 2008/05/06 07:43:08 gneumann Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 #ifdef _WIN32
@@ -362,8 +362,8 @@ NsTclMkdirObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CONST ob
         Tcl_WrongNumArgs(interp, 1, objv, "dir");
         return TCL_ERROR;
     }
-    if (mkdir(Tcl_GetString(objv[1]), 0755) != 0) {
-        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), "mkdir (\"", 
+    if (mkdir(Tcl_GetString(objv[1]), 0777) != 0) {
+       Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), "mkdir (\"", 
 		Tcl_GetString(objv[1]),
 		"\") failed:  ", Tcl_PosixError(interp), NULL);
         return TCL_ERROR;
@@ -568,6 +568,41 @@ NsTclMkTempCmd(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
 int
 NsTclTmpNamObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 {
+#ifdef WIN32
+    /*
+      The WIN32 implmentation of tmpnam() ignores the environment
+      variable TMP and generates filenames for the root
+      directory. Unfortunately, new WIN versions (Vista) don't allow
+      this. The suggested replacement is _tempnam().
+
+      The first argument of _tempnam() is the default directory, in case
+      the environment variable TMP is not set or points to a directory
+      that does not exist.
+    */
+    int i;
+    char *buf = _tempnam("/tmp", NULL);
+
+    if (buf == NULL) {
+	Tcl_SetResult(interp, "could not generate temporary filename.", TCL_STATIC);
+        return TCL_ERROR;
+    }
+    /*
+      Change back-slash characters into slash characters, as all other
+      paths are slash separated. Even some programs under Windows
+      do not allow back-slahed paths (e.g. Oracle's SqlLdr).
+    */
+    for (i = 0; i < strlen(buf); i++) {
+        if (buf[i] == '\\') buf[i] = '/';
+    }
+    /* 
+       The documentation says that _tempnam() allocates memory via
+       malloc(); to be sure, that the "right" free() is used, we do
+       not use TCL_DYNAMIC but the TCL_VOLATILE followed by the manual
+       free().
+    */
+    Tcl_SetResult(interp, buf, TCL_VOLATILE);
+    free(buf);
+#else
     char buf[L_tmpnam];
 
     if (tmpnam(buf) == NULL) {
@@ -575,6 +610,7 @@ NsTclTmpNamObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CONST o
         return TCL_ERROR;
     }
     Tcl_SetResult(interp, buf, TCL_VOLATILE);
+#endif
     return TCL_OK;
 }
 
