@@ -80,7 +80,7 @@
  *
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tcljob.c,v 1.25 2003/11/03 02:08:58 pmoosman Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tcljob.c,v 1.31 2005/08/23 21:41:31 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -144,7 +144,7 @@ typedef struct Job {
  * Queue structure. A queue manages a set of jobs.
  */
 
-typedef struct Queue {
+typedef struct JobQueue {
     char                *name;
     char                *desc;
     Ns_Mutex            lock;
@@ -155,12 +155,12 @@ typedef struct Queue {
     int                 nRunning;
     Tcl_HashTable       jobs;
     int                 refCount;
-} Queue;
-
+} JobQueue;
 
 /*
  * Thread pool structure. ns_job mananges a global set of threads.
  */
+
 typedef struct ThreadPool {
     Ns_Cond             cond;
     Ns_Mutex            queuelock;
@@ -174,28 +174,22 @@ typedef struct ThreadPool {
     Job                 *firstPtr;
 } ThreadPool;
 
-
 /*
  * Function prototypes/forward declarations.
  */
 
 static void JobThread(void *arg);
-static Job* getNextJob(void);
-
-Queue* NewQueue(CONST char* queueName, CONST char* queueDesc, int maxThreads);
-void FreeQueue(Queue *queuePtr);
-
-Job* NewJob(CONST char* server, CONST char* queueName, int type, Tcl_Obj *script);
-void FreeJob(Job *jobPtr);
-
+static Job *NextJob(void);
+static JobQueue *NewQueue(CONST char* queueName, CONST char* queueDesc, int maxThreads);
+static void FreeQueue(JobQueue *queuePtr);
+static Job* NewJob(CONST char* server, CONST char* queueName, int type, Tcl_Obj *script);
+static void FreeJob(Job *jobPtr);
 static int LookupQueue(Tcl_Interp       *interp,
                        CONST char       *queue_name,
-                       Queue            **queuePtr,
+                       JobQueue        **queuePtr,
                        int              locked);
-static int ReleaseQueue(Queue *queuePtr, int locked);
-
-static int AnyDone(Queue *queue);
-
+static int ReleaseQueue(JobQueue *queuePtr, int locked);
+static int AnyDone(JobQueue *queue);
 static CONST char* GetJobCodeStr(int code);
 static CONST char* GetJobStateStr(JobStates state);
 static CONST char* GetJobTypeStr(JobTypes type);
@@ -253,7 +247,7 @@ void
 NsTclInitQueueType(void)
 {
     Tcl_InitHashTable(&tp.queues, TCL_STRING_KEYS);
-    Ns_MutexSetName(&tp.queuelock, "threadPool");
+    Ns_MutexSetName(&tp.queuelock, "nsd:tcljobs");
     tp.nextThreadId = 0;
     tp.nextQueueId = 0;
     tp.maxThreads = 0;
@@ -355,7 +349,7 @@ int
 NsTclJobObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj **objv)
 {
     NsInterp            *itPtr = arg;
-    Queue               *queuePtr = NULL;
+    JobQueue            *queuePtr = NULL;
     Job                 *jobPtr = NULL, **nextPtrPtr;
     int                 code, new, create = 0, max;
     char                *jobId = NULL, buf[100], *queueId;
@@ -372,13 +366,13 @@ NsTclJobObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj **objv)
     enum {
         JCancelIdx, JCreateIdx, JDeleteIdx, JGenIDIdx, JJobsIdx, JJobsListIdx,
         JThreadListIdx, JQueueIdx, JQueuesIdx, JQueueListIdx, JWaitIdx, JWaitAnyIdx
-    } opt;
+    } _nsmayalias opt;
 
     if (objc < 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "option ?arg?");
 	return TCL_ERROR;
     }
-    if (Tcl_GetIndexFromObj(interp, objv[1], opts, "option", TCL_EXACT,
+    if (Tcl_GetIndexFromObj(interp, objv[1], opts, "option", 0,
                             (int *) &opt) != TCL_OK) {
         return TCL_ERROR;
     }
@@ -487,7 +481,7 @@ NsTclJobObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj **objv)
             /*
              * Create a new job and add to the Thread Pool's list of jobs.
              */
-            jobPtr = NewJob((itPtr->servPtr ? itPtr->servPtr->server : NULL),
+            jobPtr = NewJob(itPtr->servPtr->server,
                             queuePtr->name,
                             job_type,
                             objv[argIndex++]);
@@ -602,11 +596,8 @@ NsTclJobObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj **objv)
             if ((jobPtr->type == JOB_DETACHED) ||
                 (jobPtr->req == JOB_CANCEL) ||
                 (jobPtr->req == JOB_WAIT)) {
-                Tcl_AppendResult(interp,
-                                 "Cannot wait on job. Job ID : Job Req: %s",
-                                 Tcl_DStringValue(&jobPtr->id),
-                                 GetJobReqStr(jobPtr->req),
-                                 NULL);
+                Tcl_AppendResult(interp, "cannot wait on job: ",
+                    Tcl_DStringValue(&jobPtr->id), NULL);
                 ReleaseQueue(queuePtr, 0);
                 return TCL_ERROR;
             }
@@ -1057,7 +1048,7 @@ JobThread(void *arg)
     Job                 *jobPtr;
     char                buf[100];
     CONST char          *err;
-    Queue               *queuePtr;
+    JobQueue           *queuePtr;
     Tcl_HashEntry       *jPtr;
 
     Ns_WaitForStartup();
@@ -1067,7 +1058,7 @@ JobThread(void *arg)
     Ns_Log(Notice, "Starting thread: %s", buf);
     while (1) {
 	++tp.nidle;
-	while (((jobPtr = getNextJob()) == NULL) &&
+	while (((jobPtr = NextJob()) == NULL) &&
                !(tp.req == THREADPOOL_REQ_STOP)) {
 	    Ns_CondWait(&tp.cond, &tp.queuelock);
 	}
@@ -1147,10 +1138,10 @@ JobThread(void *arg)
  *----------------------------------------------------------------------
  */
 
-static Job*
-getNextJob(void)
+static Job *
+NextJob(void)
 {
-    Queue               *queuePtr;
+    JobQueue            *queuePtr;
     Tcl_HashEntry       *jPtr;
     Job                 *prev = NULL;
     Job                 *tmp = NULL;
@@ -1159,16 +1150,14 @@ getNextJob(void)
 
     jobPtr = tp.firstPtr;
     prev = tp.firstPtr;
-
     while ((!done) && (jobPtr != NULL)) {
-
         LookupQueue(NULL, jobPtr->queueId, &queuePtr, 1);
 
         /*
          * Check if the job is not cancel and 
          */
-        if (jobPtr->req == JOB_CANCEL)
-        {
+
+        if (jobPtr->req == JOB_CANCEL) {
             /*
              * Remove job from the list.
              */
@@ -1186,9 +1175,7 @@ getNextJob(void)
             jPtr = Tcl_FindHashEntry(&queuePtr->jobs, Tcl_DStringValue(&tmp->id));
             Tcl_DeleteHashEntry(jPtr);
             FreeJob(tmp);
-        }
-        else if (queuePtr->nRunning < queuePtr->maxThreads)
-        {
+        } else if (queuePtr->nRunning < queuePtr->maxThreads) {
             /*
              * Remove job from the list.
              */
@@ -1232,12 +1219,12 @@ getNextJob(void)
  *----------------------------------------------------------------------
  */
 
-Queue*
+static JobQueue *
 NewQueue(CONST char* queueName, CONST char* queueDesc, int maxThreads)
 {
-    Queue *queuePtr = NULL;
+    JobQueue *queuePtr = NULL;
 
-    queuePtr = ns_calloc(1, sizeof(Queue));
+    queuePtr = ns_calloc(1, sizeof(JobQueue));
     queuePtr->req = QUEUE_REQ_NONE;
 
     queuePtr->name = ns_calloc(1, strlen(queueName) + 1);
@@ -1274,8 +1261,8 @@ NewQueue(CONST char* queueName, CONST char* queueDesc, int maxThreads)
  *----------------------------------------------------------------------
  */
 
-void
-FreeQueue(Queue *queuePtr)
+static void
+FreeQueue(JobQueue *queuePtr)
 {
     Ns_MutexDestroy(&queuePtr->lock);
     Tcl_DeleteHashTable(&queuePtr->jobs);
@@ -1301,7 +1288,7 @@ FreeQueue(Queue *queuePtr)
  *----------------------------------------------------------------------
  */
 
-Job*
+static Job *
 NewJob(CONST char* server, CONST char* queueId, int type, Tcl_Obj *script)
 {
     Job *jobPtr = NULL;
@@ -1380,7 +1367,7 @@ FreeJob(Job *jobPtr)
  */
 static int LookupQueue(Tcl_Interp       *interp,
                        CONST char       *queueId,
-                       Queue            **queuePtr,
+                       JobQueue        **queuePtr,
                        int              locked)
 {
     Tcl_HashEntry *hPtr;
@@ -1423,7 +1410,7 @@ static int LookupQueue(Tcl_Interp       *interp,
  *----------------------------------------------------------------------
  */
 static int
-ReleaseQueue(Queue *queuePtr, int locked)
+ReleaseQueue(JobQueue *queuePtr, int locked)
 {
     Tcl_HashEntry       *qPtr;
     Tcl_HashSearch      search;
@@ -1482,7 +1469,7 @@ ReleaseQueue(Queue *queuePtr, int locked)
  */
 
 static int
-AnyDone(Queue *queue)
+AnyDone(JobQueue *queue)
 {
     Tcl_HashEntry       *hPtr;
     Job                 *jobPtr;
