@@ -27,14 +27,13 @@
  * version of this file under either the License or the GPL.
  */
 
-
 /* 
  * tclrequest.c --
  *
  *	Routines for Tcl proc and ADP registered requests.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tclrequest.c,v 1.8 2003/01/18 19:24:20 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tclrequest.c,v 1.13 2005/08/23 22:05:04 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -61,8 +60,7 @@ static Ns_FilterProc ProcFilter;
 static Proc *NewProc(char *name, char *args);
 static Ns_Callback FreeProc;
 static void AppendConnId(Tcl_DString *dsPtr, Ns_Conn *conn);
-static int RegisterFilter(NsInterp *itPtr, int when, char **argv);
-static void RegisterFilterObj(NsInterp *itPtr, int when, int objc,
+static int RegisterFilterObj(NsInterp *itPtr, int when, int objc,
 			      Tcl_Obj *CONST objv[]);
 static int GetNumArgs(Tcl_Interp *interp, Proc *procPtr);
 
@@ -138,7 +136,9 @@ badargs:
 	flags = 0;
 	idx = 1;
     }
-    server = itPtr->servPtr->server;
+    if (NsTclGetServer(itPtr, &server) != TCL_OK) {
+	return TCL_ERROR;
+    }
     method = Tcl_GetString(objv[idx++]);
     url = Tcl_GetString(objv[idx++]);
     name = Tcl_GetString(objv[idx++]);
@@ -180,7 +180,9 @@ badargs:
     if (objc == 5 && !STREQ(Tcl_GetString(objv[1]), "-noinherit")) {
 	goto badargs;
     }
-    server = itPtr->servPtr->server;
+    if (NsTclGetServer(itPtr, &server) != TCL_OK) {
+	return TCL_ERROR;
+    }
     method = Tcl_GetString(objv[objc-3]);
     url = Tcl_GetString(objv[objc-2]);
     file = ns_strdup(Tcl_GetString(objv[objc-1]));
@@ -210,7 +212,7 @@ int
 NsTclUnRegisterObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 {
     NsInterp *itPtr = arg;
-    char *server = itPtr->servPtr->server;
+    char *server;
     
     if (objc != 3 && objc != 4) {
         Tcl_WrongNumArgs(interp, 1, objv, "?-noinherit? method url");
@@ -222,6 +224,9 @@ NsTclUnRegisterObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CON
 		"\": should be -noinherit", NULL);
 	return TCL_ERROR;
     }
+    if (NsTclGetServer(itPtr, &server) != TCL_OK) {
+	return TCL_ERROR;
+    }
     Ns_UnRegisterRequest(server, Tcl_GetString(objv[objc-2]), Tcl_GetString(objv[objc-1]), 
 			objc == 3 ? 1 : 0);
     return TCL_OK;
@@ -251,36 +256,50 @@ NsTclRegisterFilterObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj
     int       lobjc;
     Tcl_Obj **lobjv;
     int       when, i;
-    char     *str;
+    static CONST char *wopt[] = {
+        "prequeue", "preauth", "postauth", "trace", NULL
+    };
+    enum {
+        PQIdx, PRIdx, POIdx, TRIdx,
+    } widx;
 
+    if (objc < 2) {
+        Tcl_WrongNumArgs(interp, 1, objv, "option ?args?");
+        return TCL_ERROR;
+    }
     if (objc != 5 && objc != 6) {
-        Tcl_WrongNumArgs(interp, 1, objv, "when method urlPattern script ?arg?");
+        Tcl_WrongNumArgs(interp, 1, objv, "when method url script ?arg?");
         return TCL_ERROR;
     }
     if (Tcl_ListObjGetElements(interp, objv[1], &lobjc, &lobjv) != TCL_OK) {
 	return TCL_ERROR;
     }
-    if (lobjc == 0) {
+    when = 0;
+    for (i = 0; i < lobjc; ++i) {
+    	if (Tcl_GetIndexFromObj(interp, lobjv[i], wopt, "when", 0,
+                (int *) &widx) != TCL_OK) {
+            return TCL_ERROR;
+    	}
+	switch (widx) {
+	case PQIdx:
+            when |= NS_FILTER_PRE_QUEUE;
+	    break;
+	case PRIdx:
+            when |= NS_FILTER_PRE_AUTH;
+	    break;
+	case POIdx:
+            when |= NS_FILTER_POST_AUTH;
+	    break;
+	case TRIdx:
+            when |= NS_FILTER_TRACE;
+	    break;
+	}
+    }
+    if (when == 0) {
 	Tcl_SetResult(interp, "blank filter when specification", TCL_STATIC);
 	return TCL_ERROR;
     }
-    when = 0;
-    for (i = 0; i < lobjc; ++i) {
-	str = Tcl_GetString(lobjv[i]);
-        if (STREQ(str, "preauth")) {
-            when |= NS_FILTER_PRE_AUTH;
-        } else if (STREQ(str, "postauth")) {
-	    when |= NS_FILTER_POST_AUTH;
-        } else if (STREQ(str, "trace")) {
-            when |= NS_FILTER_TRACE;
-        } else {
-	    Tcl_AppendResult(interp, "unknown when \"", str,
-		"\": should be preauth, postauth, or trace", NULL);
-	    return TCL_ERROR;
-        }
-    }
-    RegisterFilterObj(itPtr, when, objc - 2, objv + 2);
-    return TCL_OK;
+    return RegisterFilterObj(itPtr, when, objc - 2, objv + 2);
 }
 
 
@@ -309,7 +328,52 @@ NsTclRegisterTraceObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *
         Tcl_WrongNumArgs(interp, 1, objv, "method urlPattern script ?arg?");
         return TCL_ERROR;
     }
-    RegisterFilterObj(itPtr, NS_FILTER_VOID_TRACE, objc - 1, objv + 1);
+    return RegisterFilterObj(itPtr, NS_FILTER_VOID_TRACE, objc - 1, objv + 1);
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * NsTclRegisterFastPathObjCmd --
+ *
+ *	Implements ns_register_fastpath as obj command.
+ *
+ * Results:
+ *	Tcl result. 
+ *
+ * Side effects:
+ *	See docs. 
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+NsTclRegisterFastPathObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
+{
+    NsInterp *itPtr = arg;
+    NsServer *servPtr = itPtr->servPtr;
+    char     *server, *method, *url;
+    int       flags, idx;
+
+    if (objc < 3 || objc > 4) {
+    badargs:
+        Tcl_WrongNumArgs(interp, 1, objv, "?-noinherit? method url");
+        return TCL_ERROR;
+    }
+    flags = 0;
+    idx = 1;
+    if (objc == 4) {
+        if (!STREQ(Tcl_GetString(objv[1]), "-noinherit")) {
+            goto badargs;
+        }
+        flags = NS_OP_NOINHERIT;
+        idx++;
+    }
+    server = servPtr->server;
+    method = Tcl_GetString(objv[idx++]);
+    url = Tcl_GetString(objv[idx++]);
+    Ns_RegisterRequest(server, method, url, Ns_FastPathOp, NULL, servPtr, flags);
     return TCL_OK;
 }
 
@@ -342,13 +406,11 @@ AdpRequest(void *arg, Ns_Conn *conn)
  *
  * RequstProc --
  *
- *	Ns_OpProc for Tcl operations. Enters the header and form 
- *	connection sets, sets the current connection, concats the Tcl 
- *	command and args strings from the arg and evaluates the 
- *	resulting string. 
+ *	Ns_OpProc for ns_register_proc callbacks.  Constructs and
+ *	evaluates the given script.
  *
  * Results:
- *	NS_OK or NS_ERROR. 
+ *	Standard request result.
  *
  * Side effects:
  *	None. 
@@ -360,39 +422,45 @@ static int
 ProcRequest(void *arg, Ns_Conn *conn)
 {
     Proc	*procPtr = arg;
-    Tcl_Interp  *interp;
-    int     	 cnt;
-    Tcl_DString  cmd;
-    int          retval;
+    Tcl_Interp  *interp = Ns_GetConnInterp(conn);
+    int     	 result, cnt;
+    Tcl_DString  script;
 
-    retval = NS_OK;
-    Tcl_DStringInit(&cmd);
-    Tcl_DStringAppendElement(&cmd, procPtr->name);
-    interp = Ns_GetConnInterp(conn);
 
     /*
-     * Build the procedure arguments.  Now that we don't require the
-     * connId parameter, there are three cases to consider:
-     *   - no args -> don't add anything after the command name
-     *   - one arg -> just add the context (the arg specified at register time)
-     *   - two+ args-> (backward compatibility), the connId and the context
+     * Construct and evaluate the script. The legacy connId parameter is
+     * no longer required so the code below supports three cases:
+     *
+     * 1. No args: Append nothing.
+     * 2. One arg: Append the callback arg only.
+     * 3. Two or more args: Append the connId and callback arg.
      */
 
+    Tcl_DStringInit(&script);
+    Tcl_DStringAppendElement(&script, procPtr->name);
     cnt = GetNumArgs(interp, procPtr);
     if (cnt != 0) {
 	if (cnt > 1) {
-	    AppendConnId(&cmd, conn);
+	    AppendConnId(&script, conn);
 	}
-	Tcl_DStringAppendElement(&cmd, procPtr->args ? procPtr->args : "");
+	Tcl_DStringAppendElement(&script, procPtr->args ? procPtr->args : "");
     }
-    if (Tcl_GlobalEval(interp, cmd.string) != TCL_OK) {
+    result = Tcl_EvalEx(interp, script.string, script.length, 0);
+    Tcl_DStringFree(&script);
+
+    /*
+     * On script error, generate an internal error response if
+     * no content has been sent on the connection.
+     */
+
+    if (result != TCL_OK) {
         Ns_TclLogError(interp);
         if (Ns_ConnResetReturn(conn) == NS_OK) {
-            retval = Ns_ConnReturnInternalError(conn);
+            return Ns_ConnReturnInternalError(conn);
         }
     }
-    Tcl_DStringFree(&cmd);
-    return retval;
+
+    return NS_OK;
 }
 
 
@@ -401,10 +469,10 @@ ProcRequest(void *arg, Ns_Conn *conn)
  *
  * ProcFilter --
  *
- *	The callback for Tcl filters. Run the script. 
+ *	Callback for Tcl-based connection filters.
  *
  * Results:
- *	NS_OK, NS_FILTER_RETURN, or NS_FILTER_BREAK.
+ *	Standard filter result.
  *
  * Side effects:
  *	None. 
@@ -416,80 +484,70 @@ static int
 ProcFilter(void *arg, Ns_Conn *conn, int why)
 {
     Proc	        *procPtr = arg;
-    Tcl_DString          cmd;
-    Tcl_Interp          *interp;
-    int                  status;
-    int                  cnt;
+    Tcl_Interp          *interp = Ns_GetConnInterp(conn);
+    Tcl_DString          script;
+    int                  status, cnt;
     CONST char		*result;
 
-    Tcl_DStringInit(&cmd);
-
     /*
-     * Start building the command with the proc name
-     */
-    
-    Tcl_DStringAppendElement(&cmd, procPtr->name);
-    interp = Ns_GetConnInterp(conn);
-
-    /*
-     * Now add the optional filter arg...
+     * Construct and evaluate the script. The filter arg and legacy connId
+     * args will be appended before the "why" argument if present.
      */
     
+    Tcl_DStringInit(&script);
+    Tcl_DStringAppendElement(&script, procPtr->name);
     cnt = GetNumArgs(interp, procPtr);
     if (cnt > 1) {
 	if (cnt > 2) {
-	    AppendConnId(&cmd, conn);
+	    AppendConnId(&script, conn);
 	}
-	Tcl_DStringAppendElement(&cmd, procPtr->args ? procPtr->args : "");
+	Tcl_DStringAppendElement(&script, procPtr->args ? procPtr->args : "");
     }
-
-    /*
-     * Append the 'why'
-     */
-    
     switch (why) {
 	case NS_FILTER_PRE_AUTH:
-	    Tcl_DStringAppendElement(&cmd, "preauth");
+	    Tcl_DStringAppendElement(&script, "preauth");
 	    break;
 	case NS_FILTER_POST_AUTH:
-	    Tcl_DStringAppendElement(&cmd, "postauth");
+	    Tcl_DStringAppendElement(&script, "postauth");
 	    break;
 	case NS_FILTER_TRACE:
-	    Tcl_DStringAppendElement(&cmd, "trace");
+	    Tcl_DStringAppendElement(&script, "trace");
+	    break;
+	default:
+	    /* NB: Do not append a why argument. */
 	    break;
     }
-
-    /*
-     * Run the script.
-     */
-    
     Tcl_AllowExceptions(interp);
-    status = Tcl_GlobalEval(interp, cmd.string);
+    status = Tcl_EvalEx(interp, script.string, script.length, 0);
+    Tcl_DStringFree(&script);
     if (status != TCL_OK) {
 	Ns_TclLogError(interp);
     }
 
     /*
-     * Determine the filter result code.
+     * Determine the filter return code from the Tcl result string.
      */
 
-    result = Tcl_GetStringResult(interp);
     if (why == NS_FILTER_VOID_TRACE) {
+	/* NB: Result string ignored for traces. */
 	status = NS_OK;
     } else if (status != TCL_OK) {
+	/* NB: Filter error on script error. */
 	status = NS_ERROR;
-    } else if (STREQ(result, "filter_ok")) {
-	status = NS_OK;
-    } else if (STREQ(result, "filter_break")) {
-	status = NS_FILTER_BREAK;
-    } else if (STREQ(result, "filter_return")) {
-	status = NS_FILTER_RETURN;
     } else {
-	Ns_Log(Warning, "tclfilter: %s return invalid result: %s",
-	    procPtr->name, result);
-	status = NS_ERROR;
+    	result = Tcl_GetStringResult(interp);
+        if (STREQ(result, "filter_ok")) {
+	    status = NS_OK;
+    	} else if (STREQ(result, "filter_break")) {
+	    status = NS_FILTER_BREAK;
+    	} else if (STREQ(result, "filter_return")) {
+	    status = NS_FILTER_RETURN;
+    	} else {
+	    Ns_Log(Warning, "tclfilter: %s return invalid result: %s",
+	    	procPtr->name, result);
+	    status = NS_ERROR;
+	}
     }
-    Tcl_DStringFree(&cmd);
     return status;
 }
 
@@ -539,74 +597,44 @@ GetNumArgs(Tcl_Interp *interp, Proc *procPtr)
 /*
  *----------------------------------------------------------------------
  *
- * RegisterFilter --
+ * RegisterFilterObj --
  *
  *	Register a Tcl filter. 
  *
  * Results:
- *	TCL_OK. 
+ *	TCL_ERROR if no server, TCL_OK otherwise. 
  *
  * Side effects:
- *	Will allocate memory for TclContext as well as strdup all the 
- *	arguments. 
+ *	Will register a filter to run later.
  *
  *----------------------------------------------------------------------
  */
 
 static int
-RegisterFilter(NsInterp *itPtr, int when, char **argv)
+RegisterFilterObj(NsInterp *itPtr, int when, int objc, Tcl_Obj *CONST objv[])
 {
     Proc	    *procPtr;
-    char	    *server, *method, *url;
+    char	    *server, *method, *url, *name, *args;
 
-    server = itPtr->servPtr->server;
-    method = argv[0];
-    url = argv[1];
-    procPtr = NewProc(argv[2], argv[3]);
+    if (NsTclGetServer(itPtr, &server) != TCL_OK) {
+	return TCL_ERROR;
+    }
+    method = Tcl_GetString(objv[0]);
+    url = Tcl_GetString(objv[1]);
+    name = Tcl_GetString(objv[2]);
+    args = (objc > 3 ? Tcl_GetString(objv[3]) : NULL);
+    procPtr = NewProc(name, args);
     Ns_RegisterFilter(server, method, url, ProcFilter, when, procPtr);
     return TCL_OK;
 }
 
-
-/*
- *----------------------------------------------------------------------
- *
- * RegisterFilterObj --
- *
- *	Register a Tcl filter. 
- *
- * Results:
- *	TCL_OK. 
- *
- * Side effects:
- *	Will allocate memory for TclContext as well as strdup all the 
- *	arguments. 
- *
- *----------------------------------------------------------------------
- */
-
-static void
-RegisterFilterObj(NsInterp *itPtr, int when, int objc, Tcl_Obj *CONST objv[])
-{
-    Proc	    *procPtr;
-    char	    *server, *method, *url, *name, *args;
-
-    server = itPtr->servPtr->server;
-    method = Tcl_GetString(objv[0]);
-    url = Tcl_GetString(objv[1]);
-    name = Tcl_GetString(objv[2]);
-    args = (objc > 3 ? Tcl_GetString(objv[3]) : NULL);
-    procPtr = NewProc(name, args);
-    Ns_RegisterFilter(server, method, url, ProcFilter, when, procPtr);
-}
-
 
 /*
  *----------------------------------------------------------------------
  *
  * AppendConnId --
  *
- *	Append the tcl conn handle to a dstring. 
+ *	Append the Tcl conn handle to a dstring. 
  *
  * Results:
  *	None. 
