@@ -34,7 +34,7 @@
  *	Tcl commands that let you do TCP sockets. 
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tclsock.c,v 1.17.2.1 2004/12/06 16:33:43 dossy Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tclsock.c,v 1.24 2005/08/02 22:11:58 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsd.h"
 
@@ -42,12 +42,12 @@ static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsd/tclso
  * The following structure is used for a socket callback.
  */
 
-typedef struct Callback {
+typedef struct TclSockCallback {
     char       *server;
     Tcl_Channel chan;
     int    	when;
     char        script[1];
-} Callback;
+} TclSockCallback;
 
 /*
  * The following structure is used for a socket listen callback.
@@ -69,8 +69,6 @@ static void AppendReadyFiles(Tcl_Interp *interp, fd_set * pset, int write,
 static int EnterSock(Tcl_Interp *interp, SOCKET sock);
 static int EnterDup(Tcl_Interp *interp, SOCKET sock);
 static int EnterDupedSocks(Tcl_Interp *interp, SOCKET sock);
-static int SockSetBlocking(char *value, Tcl_Interp *interp, int argc,
-			   	char **argv);
 static int SockSetBlockingObj(char *value, Tcl_Interp *interp, int objc, 
 				Tcl_Obj *CONST objv[]);
 static Ns_SockProc SockListenCallback;
@@ -78,7 +76,7 @@ static Ns_SockProc SockListenCallback;
 void
 NsTclSockArgProc(Tcl_DString *dsPtr, void *arg)
 {
-    Callback *cbPtr = arg;
+    TclSockCallback *cbPtr = arg;
 
     Tcl_DStringAppendElement(dsPtr, cbPtr->script);
 }
@@ -392,71 +390,125 @@ NsTclSockCheckObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CONS
  */
 
 int
-NsTclSockOpenObjCmd(ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
+NsTclSockOpenObjCmd(
+    ClientData arg, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]
+)
 {
     SOCKET sock;
-    int port;
-    int timeout;
-    int first;
-    int async;
+    char *host, *lhost = NULL, *opt, *val;
+    int lport = 0, port, first, async = 0, timeout = -1;
 
-    if (objc < 3 || objc > 5) {
-        Tcl_WrongNumArgs(interp, 1, objv, "?-nonblock|-timeout seconds? host port");
+    if (objc < 3 || objc > 9) {
+      syntax:
+        Tcl_WrongNumArgs(interp, 1, objv,
+                "?(-nonblock | -async) | -timeout seconds? "
+                "?-localhost host? ?-localport port? host port");
         return TCL_ERROR;
     }
-    first = 1;
-    async = 0;
-    timeout = -1;
-    if (objc == 4) {
+    
+    /*
+     * Parse optional arguments.  Note that either the:
+     *     -nonblock | -async
+     * or
+     *     -timeout seconds
+     * combinations are accepted.
+     */
 
-	/*
-	 * open -nonblock host port
-	 */
-	
-        if (!STREQ(Tcl_GetString(objv[1]), "-nonblock") && 
-	    !STREQ(Tcl_GetString(objv[1]), "-async")) {
-	    Tcl_WrongNumArgs(interp, 1, objv, "?-nonblock|-timeout seconds? host port");
-	   return TCL_ERROR;
+    for (first = 1; first < objc; first++) {
+        opt= Tcl_GetString(objv[first]);
+        if (*opt != '-') {
+            break; /* End of options */
         }
-
-        first = 2;
-        async = 1;
-    } else if (objc == 5) {
-
-	/*
-	 * open -timeout seconds host port
-	 */
-
-        if (!STREQ(Tcl_GetString(objv[1]), "-timeout")) {
-        	Tcl_WrongNumArgs(interp, 1, objv, "?-nonblock|-timeout seconds? host port");
-	    	return TCL_ERROR;
+        if (STREQ(opt, "-nonblock") || STREQ(opt, "-async")) {
+            if (timeout >= 0) {
+                goto syntax;
+            }
+            async = 1;
+        } else if (STREQ(opt, "-localhost")) {
+            if (++first >= objc) {
+                goto syntax;
+            }
+            lhost = Tcl_GetString(objv[first]);
+            if (*lhost == 0) {
+                Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+                        "invalid hostname: must not be empty", NULL);
+                return TCL_ERROR;
+            }
+        } else if (STREQ(opt, "-timeout")) {
+            if (++first >= objc || async) {
+                goto syntax;
+            }
+            if (Tcl_GetIntFromObj(interp, objv[first], &timeout) != TCL_OK) {
+                return TCL_ERROR;
+            }
+        } else if (STREQ(opt, "-localport")) {
+            if (++first >= objc) {
+                goto syntax;
+            }
+            if (Tcl_GetIntFromObj(interp, objv[first], &lport) != TCL_OK) {
+                return TCL_ERROR;
+            }
+            if (lport < 0) {
+                val = Tcl_GetString(objv[first]);
+                Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+                        "invalid port: ", val, "; must be > 0", NULL);
+                return TCL_ERROR;
+            }
+        } else {
+            Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+                    "invalid option: \"", opt, "\"", NULL);
+            return TCL_ERROR;   
         }
-        if (Tcl_GetIntFromObj(interp, objv[2], &timeout) != TCL_OK) {
-            return TCL_ERROR;
-        }
-        first = 3;
     }
-    if (Tcl_GetIntFromObj(interp, objv[first + 1], &port) != TCL_OK) {
+
+    if ((objc - first) != 2) {
+        goto syntax;
+    }
+
+    /*
+     * Get the host to connect to. Bark on invalid entry.
+     */
+
+    host = Tcl_GetString(objv[first]);
+    if (*host == 0) {
+        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+                "invalid hostname: must not be empty", NULL);
+        return TCL_ERROR;
+    }
+
+    /*
+     * Get the port to connect to. Bark on invalid entry.
+     */
+
+    if (Tcl_GetIntFromObj(interp, objv[first+1], &port) != TCL_OK) {
+        return TCL_ERROR;
+    } else if (port < 0) {
+        val = Tcl_GetString(objv[first+1]);
+        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+                "invalid port: ", val, "; must be > 0", NULL);
         return TCL_ERROR;
     }
 
     /*
      * Perform the connection.
      */
-    
+
     if (async) {
-        sock = Ns_SockAsyncConnect(Tcl_GetString(objv[first]), port);
+        sock = Ns_SockAsyncConnect2(host, port, lhost, lport);
     } else if (timeout < 0) {
-        sock = Ns_SockConnect(Tcl_GetString(objv[first]), port);
+        sock = Ns_SockConnect2(host, port, lhost, lport);
     } else {
-        sock = Ns_SockTimedConnect(Tcl_GetString(objv[first]), port, timeout);
+        sock = Ns_SockTimedConnect2(host, port, lhost, lport, timeout);
     }
+
     if (sock == INVALID_SOCKET) {
-	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp), "could not connect to \"",
-		Tcl_GetString(objv[first]), ":", 
-		Tcl_GetString(objv[first + 1]), "\"", NULL);
+        char *why = Tcl_GetErrno() ? Tcl_PosixError(interp) : "reason unknown";
+        Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
+                "can't connect to \"", host, ":",
+                Tcl_GetString(objv[first+1]), "\"; ", why, NULL);
         return TCL_ERROR;
     }
+
     return EnterDupedSocks(interp, sock);
 }
 
@@ -672,7 +724,7 @@ NsTclSockCallbackObjCmd(ClientData arg, Tcl_Interp *interp, int objc,
     SOCKET sock;
     int     when;
     char   *s;
-    Callback *cbPtr;
+    TclSockCallback *cbPtr;
     NsInterp *itPtr = arg;
 
     if (objc != 4) {
@@ -720,8 +772,8 @@ NsTclSockCallbackObjCmd(ClientData arg, Tcl_Interp *interp, int objc,
      */
 
     sock = ns_sockdup(sock);
-    cbPtr = ns_malloc(sizeof(Callback) + Tcl_GetCharLength(objv[2]));
-    cbPtr->server = (itPtr->servPtr ? itPtr->servPtr->server : NULL);
+    cbPtr = ns_malloc(sizeof(TclSockCallback) + Tcl_GetCharLength(objv[2]));
+    cbPtr->server = itPtr->servPtr->server;
     cbPtr->chan = NULL;
     cbPtr->when = when;
     strcpy(cbPtr->script, Tcl_GetString(objv[2]));
@@ -774,7 +826,7 @@ NsTclSockListenCallbackObjCmd(ClientData arg, Tcl_Interp *interp, int objc,
         addr = NULL;
     }
     lcbPtr = ns_malloc(sizeof(ListenCallback) + Tcl_GetCharLength(objv[3]));
-    lcbPtr->server = (itPtr->servPtr ? itPtr->servPtr->server : NULL);
+    lcbPtr->server = itPtr->servPtr->server;
     strcpy(lcbPtr->script, Tcl_GetString(objv[3]));
     if (Ns_SockListenCallback(addr, port, SockListenCallback, lcbPtr) != NS_OK) {
 	Tcl_SetResult(interp, "could not register callback", TCL_STATIC);
@@ -784,40 +836,6 @@ NsTclSockListenCallbackObjCmd(ClientData arg, Tcl_Interp *interp, int objc,
     return TCL_OK;
 }
 
-
-/*
- *----------------------------------------------------------------------
- *
- * SockSetBlocking --
- *
- *	Set a socket blocking. 
- *
- * Results:
- *	Tcl result. 
- *
- * Side effects:
- *	None. 
- *
- *----------------------------------------------------------------------
- */
-
-static int
-SockSetBlocking(char *value, Tcl_Interp *interp, int argc, char **argv)
-{
-    Tcl_Channel chan;
-
-    if (argc != 2) {
-        Tcl_AppendResult(interp, "wrong # args: should be \"",
-            argv[0], " sockId\"", NULL);
-        return TCL_ERROR;
-    }
-    chan = Tcl_GetChannel(interp, argv[1], NULL);
-    if (chan == NULL) {
-	return TCL_ERROR;
-    }
-    return Tcl_SetChannelOption(interp, chan, "-blocking", value);
-}
-
 
 /*
  *----------------------------------------------------------------------
@@ -958,8 +976,8 @@ GetSet(Tcl_Interp *interp, char *flist, int write, fd_set **setPtrPtr,
             status = TCL_ERROR;
             break;
         }
-        if (sock > *maxPtr) {
-            *maxPtr = sock;
+        if (sock > (SOCKET) *maxPtr) {
+            *maxPtr = (int) sock;
         }
         FD_SET(sock, setPtr);
     }
@@ -1051,7 +1069,7 @@ NsTclSockProc(SOCKET sock, void *arg, int why)
     Tcl_Obj	*objPtr;
     char        *w;
     int          result, ok;
-    Callback    *cbPtr = arg;
+    TclSockCallback    *cbPtr = arg;
 
     if (why != NS_SOCK_EXIT || (cbPtr->when & NS_SOCK_EXIT)) {
 	Tcl_DStringInit(&script);
