@@ -33,17 +33,15 @@
  *	Database module entry point.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsdb/nsdb.c,v 1.1 2002/06/05 22:57:41 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsdb/nsdb.c,v 1.3 2005/08/08 11:32:17 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "db.h"
 
-int Ns_ModuleVersion = 1;
-
 
 /*
  *----------------------------------------------------------------------
  *
- * Ns_ModuleInit --
+ * NsDb_ModInit --
  *
  *	Module initialization point.
  *
@@ -57,10 +55,14 @@ int Ns_ModuleVersion = 1;
  */
 
 int
-Ns_ModuleInit(char *server, char *module)
+NsDb_ModInit(char *server, char *module)
 {
     static int once;
 
+    if (server == NULL) {
+	Ns_Log(Error, "%s: attempt to load outside a virual server", module);
+	return NS_ERROR;
+    }
     if (!once) {
 	NsDbInitPools();
 	once = 1;
