@@ -34,7 +34,7 @@
  *	This file implements the access log using NCSA Common Log format.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nslog/nslog.c,v 1.14.2.1 2004/07/13 21:01:14 dossy Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nslog/nslog.c,v 1.16 2005/08/08 11:32:18 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "ns.h"
 #include <sys/stat.h>	/* mkdir */
@@ -44,12 +44,6 @@ static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nslog/nsl
 #define LOG_FMTTIME	2
 #define LOG_REQTIME	4
 
-/*
- * Exported variables
- */
-
-int Ns_ModuleVersion = 1; 		/* Needed for AOLserver */
-
 typedef struct {
     char	   *module;
     Ns_Mutex	    lock;
@@ -81,15 +75,11 @@ static Ns_ArgProc LogArg;
 static Tcl_CmdProc LogCmd;
 static Ns_TclInterpInitProc AddCmds;
 
-/*
- * Static variables defined in this file
- */
-
 
 /*
  *----------------------------------------------------------------------
  *
- * Ns_ModuleInit --
+ * NsLog_ModInit --
  *
  *	Module initialization routine.
  *
@@ -105,7 +95,7 @@ static Ns_TclInterpInitProc AddCmds;
  */
 
 int
-Ns_ModuleInit(char *server, char *module)
+NsLog_ModInit(char *server, char *module)
 {
     char 	*path;
     int 	 opt, hour;
