@@ -33,7 +33,7 @@
  *	Library for ns_proxy commands and main loops.
  */
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsproxy/nsproxylib.c,v 1.5 2006/06/26 00:28:30 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsproxy/nsproxylib.c,v 1.7 2008/06/20 08:06:33 gneumann Exp $, compiled: " __DATE__ " " __TIME__;
 
 #include "nsproxy.h"
 #include <poll.h>
@@ -280,7 +280,7 @@ Ns_ProxyMain(int argc, char **argv, Tcl_AppInitProc *init)
 {
     Tcl_Interp *interp;
     Proc proc;
-    int result, len, n, max;
+    int result, len, n, max = 0;
     Req *reqPtr;
     Tcl_DString in, out;
     char *script, *active, *dots;
@@ -329,7 +329,7 @@ Ns_ProxyMain(int argc, char **argv, Tcl_AppInitProc *init)
     interp = Ns_TclCreateInterp();
     if (init != NULL) {
 	if ((*init)(interp) != TCL_OK) {
-	    FatalExit(interp->result);
+	    FatalExit(Tcl_GetStringResult(interp));
 	}
     }
 
@@ -837,7 +837,7 @@ GetObjCmd(ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj **objv)
 		} else {
 		    proxyPtr = ns_calloc(1, sizeof(Proxy));
 		    proxyPtr->poolPtr = poolPtr;
-		    sprintf(proxyPtr->id, "proxy%d", poolPtr->nextid++);
+                    sprintf(proxyPtr->id, "%s-proxy-%d", poolPtr->name, poolPtr->nextid++);
 		    Tcl_DStringInit(&proxyPtr->in);
 		}
 		proxyPtr->nextPtr = firstPtr;
@@ -1460,7 +1460,7 @@ static void
 Export(Tcl_Interp *interp, int code, Tcl_DString *dsPtr)
 {
     Res hdr;
-    char *einfo, *ecode, *result;
+    char *einfo = NULL, *ecode = NULL, *result;
     int   clen, ilen, rlen;
 
     clen = ilen = rlen = 0;
@@ -1898,9 +1898,8 @@ FatalExit(char *func)
 static char *
 ProxyError(Tcl_Interp *interp, Err err)
 {
-    char *msg, *sysmsg, *code;
+    char *msg = NULL, *sysmsg = NULL, *code = NULL;
 
-    sysmsg = NULL;
     switch (err) {
     case ENone:
 	code = "ENone";
