@@ -48,7 +48,7 @@
 #define PTHREAD_TEST 1
 #endif
 
-static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsthread/nsthreadtest.c,v 1.4 2003/06/02 14:42:00 vasiljevic Exp $, compiled: " __DATE__ " " __TIME__;
+static const char *RCSID = "@(#) $Header: /cvsroot/aolserver/aolserver/nsthread/nsthreadtest.c,v 1.6 2005/08/08 11:30:51 jgdavidson Exp $, compiled: " __DATE__ " " __TIME__;
 
 /*
  * Collection of synchronization objects for tests.
@@ -118,11 +118,9 @@ TlsLogArg(void *arg)
 int
 RecursiveStackCheck(int n)
 {
-#if 0
     if (Ns_CheckStack() == NS_OK) {
 	n = RecursiveStackCheck(n);
     }
-#endif
     ++n;
     return n;
 }
@@ -132,6 +130,7 @@ CheckStackThread(void *arg)
 {
     int n;
 
+    Ns_ThreadSetName("checkstack");
     n = RecursiveStackCheck(0);
     Ns_ThreadExit((void *) n);
 }
@@ -238,6 +237,7 @@ MemThread(void *arg)
     int             i;
     void           *ptr;
 
+    Ns_ThreadSetName("memthread");
     Ns_MutexLock(&lock);
     ++nrunning;
     Ns_CondBroadcast(&cond);
@@ -342,6 +342,18 @@ DumperThread(void *arg)
     Ns_MutexUnlock(&block);
 }
 
+
+static void
+DetachedThread(void *ignored)
+{
+    int i;
+
+    for (i = 0; i < 10; ++i) {
+	Msg("\n\ndetached! %d", i);
+	sleep(1);
+    }
+}
+
 #if PTHREAD_TEST
 
 /*
@@ -371,6 +383,8 @@ Pthread(void *arg)
      * self-initialization style.
      */
 
+    Ns_ThreadSetName("pthread");
+    sleep(5);
     if (tls == NULL) {
 	Ns_MasterLock();
 	if (tls == NULL) {
@@ -413,11 +427,7 @@ int main(int argc, char *argv[])
     pthread_t tids[10];
 #endif
 
-#if (STATIC_BUILD == 1)
-    extern void NsthreadsInit();
-    NsthreadsInit();
-#endif
-
+    NsThreads_LibInit();
     Ns_ThreadSetName("-main-");
 
     /*
@@ -436,6 +446,7 @@ int main(int argc, char *argv[])
 	}
     }
 
+    Ns_ThreadCreate(DetachedThread, NULL, 0, NULL);
     Ns_ThreadCreate(DumperThread, NULL, 0, &dumper);
     Ns_MutexSetName(&lock, "startlock");
     Ns_MutexSetName(&dlock, "dumplock");
