@@ -32,14 +32,20 @@
  *
  *	Private nsthread library include.
  *
- *	$Header: /cvsroot/aolserver/aolserver/nsthread/thread.h,v 1.4 2003/01/18 19:56:30 jgdavidson Exp $
+ *	$Header: /cvsroot/aolserver/aolserver/nsthread/thread.h,v 1.8 2005/08/23 21:41:31 jgdavidson Exp $
  */
 
 #ifndef THREAD_H
 #define THREAD_H
 
+#define NSTHREAD_EXPORTS
 #include "nsthread.h"
 
+#ifdef WIN32
+typedef char *caddr_t;
+#endif
+
+extern int    NsGetStack(void **addrPtr, size_t *sizePtr);
 extern void   NsthreadsInit(void);
 extern void   NsInitThreads(void);
 extern void   NsInitMaster(void);
@@ -55,6 +61,6 @@ extern void   NsCleanupTls(void **slots);
 extern void **NsGetTls(void);
 extern void   NsThreadMain(void *arg);
 extern void   NsCreateThread(void *arg, long stacksize, Ns_Thread *threadPtr);
-extern void   NsThreadFatal(char *func, char *osfunc, int err);
+extern void   NsThreadFatal(char *func, char *osfunc, int err) _nsnoreturn;
 
 #endif /* THREAD_H */
